package edu.northwestern.cs.aqualab.cattle;

import edu.northwestern.cs.aqualab.cattle.jsonsimple.JSONObject;



/**
 * Represents the outcome of an attempted delivery of a single Report. success
 * represents end-to-end success and will be true if and only if cattle received
 * a response for the server indicating acceptance of the report. <br>
 * <br>
 * Regardless of success, if the delivery attempt made it far enough to get a response from the
 * server, it will be available in the response field. <br>
 * <br>
 * To determine what went wrong with a failed report, examine, in this order: <br>
 * <ul>
 * <li>exception, for client-side and network errors</li>
 * <li>HTTP_code, for problems with wrangler's underlying server and HTTP errors (e.g., bad resource)</li>
 * <li>response, for errors at the wrangler protocol level</li>
 * </ul>
 */
public class ReportResult {
    public static int HTTP_STATUS_NEVER_SENT = -1;

    public boolean success = false;
    public int HTTPStatus = HTTP_STATUS_NEVER_SENT;
    public JSONObject response = null;
    public Exception exception;


    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("success: ");
        sb.append(this.success);

        if (this.exception != null) {
            sb.append(", exception: ");
            sb.append(exception.toString());
        } else {
            sb.append(", HTTP_CODE: ");
            sb.append(this.HTTPStatus);
            if (this.HTTPStatus == 200) {
                sb.append(", response: ");
                if (this.response == null) {
                    sb.append("null");
                } else {
                    sb.append(response.toString());
                }
            }
        }
        return sb.toString();
    }
}
