package edu.northwestern.cs.aqualab.cattle;

import java.util.regex.Pattern;



/**
 * A collection of static methods for validating simple parameters such as
 * hostnames and port numbers.
 */
public class Validate {
    private static final Pattern RE_HOST = Pattern.compile("^([A-Za-z0-9\\-]\\.?)+$");
    private static final Pattern RE_PATH_COMPONENT = Pattern.compile("^[A-Za-z0-9]+$");


    /**
     * @return Whether host appears to be a valid hostname.
     */
    public static boolean validateHost(String host) {
        return RE_HOST.matcher(host).matches();
    }


    /**
     * @return Whether port is a valid TCP/UDP port.
     */
    public static boolean validatePort(int port) {
        return (port > 0) && (port < Math.pow(2, 16));
    }


    /**
     * @return Whether project is a valid project name. This means matching the
     *         regex <code>^[A-Za-z0-9]+$</code>.
     */
    public static boolean validateProject(String project) {
        return RE_PATH_COMPONENT.matcher(project).matches();
    }


    /**
     * @return Whether dataset is a valid dataset name. Datasets and projects
     *         have the same restrictions.
     */
    public static boolean validateDataset(String dataset) {
        return validateProject(dataset);
    }
}
