package edu.northwestern.cs.aqualab.cattle.socket;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;



public class SelfSignedSSLSocketFactory extends SSLSocketFactory {
    private SSLContext mSSLContext = SSLContext.getInstance("TLS");


    public SelfSignedSSLSocketFactory(List<KeyStore> ks) throws NoSuchAlgorithmException,
                                                  KeyManagementException {
        super();
        mSSLContext.init(null, new TrustManager[] { new SelfSignedTrustManager(ks) }, null);
    }


    @Override
    public Socket createSocket(Socket s, String host, int port, boolean autoClose)
        throws IOException {
        return mSSLContext.getSocketFactory().createSocket(s, host, port, autoClose);
    }


    @Override
    public Socket createSocket(String host, int port) throws IOException, UnknownHostException {
        return mSSLContext.getSocketFactory().createSocket(host, port);
    }


    @Override
    public Socket createSocket(InetAddress host, int port) throws IOException {
        return mSSLContext.getSocketFactory().createSocket(host, port);
    }


    @Override
    public Socket createSocket(String host, int port, InetAddress localHost, int localPort)
        throws IOException, UnknownHostException {
        return mSSLContext.getSocketFactory().createSocket(host, port, localHost, localPort);
    }


    @Override
    public Socket createSocket(InetAddress address, int port, InetAddress localAddress,
                               int localPort) throws IOException {
        return mSSLContext.getSocketFactory().createSocket(address, port, localAddress, localPort);
    }


    @Override
    public String[] getDefaultCipherSuites() {
        return mSSLContext.getSocketFactory().getDefaultCipherSuites();
    }


    @Override
    public String[] getSupportedCipherSuites() {
        return mSSLContext.getSocketFactory().getSupportedCipherSuites();
    }

    public static class SelfSignedTrustManager implements X509TrustManager {
        protected ArrayList<X509TrustManager> x509TrustManagers = new ArrayList<X509TrustManager>();


        protected SelfSignedTrustManager(List<KeyStore> additionalkeyStores) {
            final ArrayList<TrustManagerFactory> factories = new ArrayList<TrustManagerFactory>();

            try {
                // The default Trustmanager with default keystore
                final TrustManagerFactory original =
                    TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
                original.init((KeyStore) null);
                factories.add(original);

                for (KeyStore keyStore : additionalkeyStores) {
                    final TrustManagerFactory additionalCerts =
                        TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
                    additionalCerts.init(keyStore);
                    factories.add(additionalCerts);
                }

            } catch (Exception e) {
                throw new RuntimeException(e);
            }

            /*
             * Iterate over the returned trustmanagers, and hold on to any that
             * are X509TrustManagers
             */
            for (TrustManagerFactory tmf : factories)
                for (TrustManager tm : tmf.getTrustManagers())
                    if (tm instanceof X509TrustManager)
                        x509TrustManagers.add((X509TrustManager) tm);

            if (x509TrustManagers.size() == 0)
                throw new RuntimeException("Couldn't find any X509TrustManagers");

        }


        /*
         * Delegate to the default trust manager.
         */
        public void checkClientTrusted(X509Certificate[] chain, String authType)
            throws CertificateException {
            final X509TrustManager defaultX509TrustManager = x509TrustManagers.get(0);
            defaultX509TrustManager.checkClientTrusted(chain, authType);
        }


        /*
         * Loop over the trustmanagers until we find one that accepts our server
         */
        public void checkServerTrusted(X509Certificate[] chain, String authType)
            throws CertificateException {

            for (X509TrustManager tm : x509TrustManagers) {
                if (tm.getAcceptedIssuers().length == 0) continue;

                try {
                    tm.checkServerTrusted(chain, authType);
                    return;
                } catch (CertificateException e) {
                    // ignore
                }
            }
            throw new CertificateException();
        }


        public X509Certificate[] getAcceptedIssuers() {
            final ArrayList<X509Certificate> list = new ArrayList<X509Certificate>();
            for (X509TrustManager tm : x509TrustManagers)
                list.addAll(Arrays.asList(tm.getAcceptedIssuers()));
            return list.toArray(new X509Certificate[list.size()]);
        }
    }
}
