/**
 * Dasu Project
 *
 * File:         DasuHttpModule.java
 * RCS:          $Id$
 * Description:  DasuHttpModule class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 15, 2010 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.ono.experiment
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.dynamic.measurement;

import java.io.IOException;
import java.net.InetAddress;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;

import org.xbill.DNS.*; 

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactDnsTimeout;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.dynamic.drools.FactDnsProbeResult;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.DasuProbeModule;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The DasuScrapeModule class manages the set of 
 *         Scrapes performed by the peer.
 */

public class DasuDnsModule extends DasuProbeModule<FactDnsProbeResult> {	
	
	// XXX: need to finish hooking up probesubtask!!! This will require work in AMC, and how it calls the probe* functions.		
	private static boolean DEBUG = false;
	
	ArrayList<ArrayList<String>> myDns =  null;
	private static DasuDnsModule self;
	private Hashtable<String, Long> timeoutIsRunning;	
	
	public DasuDnsModule() {		
		super("DasuDnsModule", 2.0f);
		timeoutIsRunning = new Hashtable<String, Long>();
	}
	
	public synchronized static DasuDnsModule getInstance() {
		
		if (self == null) {
			self = new DasuDnsModule();
			self.setDaemon(true);			
		}		
		return self;
				
	}
	
	public boolean isIdle() {
		return idle;
	}
	
	public String getType(String options) {
		
		String type = "A";
		
		if (options != null) {
			int arg = 0;
			String[] argv = options.split(" ");
			while (argv.length > arg && argv[arg].startsWith("-") && (argv[arg].length() > 1)) {
	
				switch (argv[arg].charAt(1)) {
	
				case 't':
					
					String typeString;
					
					if (argv[arg].length() > 2) {
						typeString = argv[arg].substring(2);
					} else {
						typeString = argv[++arg];
					}
	
					type = typeString;
					
					break;
				
				case 'x':
					
					type = "PTR";
					break;
				}
				
				arg++;
			}
		}
		
		return type;
		
	}
	
	@Override
	protected FactDnsProbeResult probeIp(ProbeSubTask pst) {
		
		if ( !isActive() )
			return null;
		
		String server = null;
		if(pst.getParams() != null && pst.getParams().containsKey("server")) 
			server = (String) pst.getParams().get("server");

		String options = null;
		if(pst.getParams() != null && pst.getParams().containsKey("options")) 
			options = (String) pst.getParams().get("options");
		
		if ( server == null ) {
			org.xbill.DNS.ResolverConfig DNSConfig = new org.xbill.DNS.ResolverConfig();
			server = DNSConfig.server();					
		}
		
		Long requestTime = Util.currentGMTTime();
		Message msg = doLookup(pst.getToProbe(), server, options, pst.getTaskId());
		
		if (DEBUG)
			LOGGER.info("DNS: msg: " + msg);

		boolean timeout = (( msg != null ) && ( msg.getHeader().getID() != -1)) ? false : true;
		msg = ( msg == null) ? new Message() : msg; 

		FactDnsProbeResult ret = new FactDnsProbeResult(server, pst.getToProbe(), options,
				requestTime, msg, Util.currentGMTTime(), getType(options), pst.getTaskId());
		ret.setTarget(pst.getTarget());		
		ret.setTimeout(timeout);		
		reportProbeResult(ret);
		
		ret.setPst(pst);
		return ret;

	}
	
	// TODO: merge with Util.doLookup.
	// JSO: not sure how this code differs from Util.doLookup, but a diff shows that it does.
	public Message doLookup(String nameString, String server, String args, Integer taskId) {

		String[] argv = new String[]{};
		if (args != null)
			argv = args.split(" ");
		
		org.xbill.DNS.Name name = null;
		int type = org.xbill.DNS.Type.A;
		int dclass = DClass.IN;
		int arg;
		Message query;
		Message response = null;
		Record rec;
		SimpleResolver res = null;
		boolean printQuery = false;
		
		if ( server == null) {
			LOGGER.info("DNS: server is null");
			return null;
		}

		if (timeoutIsRunning.containsKey(server)) {
			return null;			
		}

		try {
			
			res = new SimpleResolver(server);
			
			arg = 0;
			name = Name.fromString(nameString, Name.root);

			if ( argv.length > 0) {
				
				while (argv.length > arg && argv[arg].startsWith("-") && (argv[arg].length() > 1)) {
					
					switch (argv[arg].charAt(1)) {

					case 'x':
						name = ReverseMap.fromAddress(nameString);
						type = Type.PTR;
						dclass = DClass.IN;
						break;
						
					case 'p':
	
						String portStr;
						int port;
	
						if (argv[arg].length() > 2) {
							portStr = argv[arg].substring(2);
						} else {
							portStr = argv[++arg];
						}
	
						port = Integer.parseInt(portStr);
	
						if ((port < 0) || (port > 65536)) {
							LOGGER.info("DNS: Invalid port");
	
							return null;
						}
	
						res.setPort(port);
	
						break;
	
					case 'b':
	
						String addrStr;
	
						if (argv[arg].length() > 2) {
							addrStr = argv[arg].substring(2);
						} else {
							addrStr = argv[++arg];
						}
	
						InetAddress addr;
	
						try {
							addr = InetAddress.getByName(addrStr);
						} catch (Exception e) {
							LOGGER.info("DNS: Invalid address");
	
							return null;
						}
	
						res.setLocalAddress(addr);
	
						break;
	
					case 'k':
	
						String key;
	
						if (argv[arg].length() > 2) {
							key = argv[arg].substring(2);
						} else {
							key = argv[++arg];
						}
	
						res.setTSIGKey(TSIG.fromString(key));
	
						break;
	
					case 'T':
	
						res.setTCP(true);
						break;
						
					case 't':
						
						String typeString;
						
						if (argv[arg].length() > 2) {
							typeString = argv[arg].substring(2);
						} else {
							typeString = argv[++arg];
						}
	
						type = Type.value(typeString);
						
						break;
	
					case 'i':
						res.setIgnoreTruncation(true);
	
						break;
	
					case 'e':
	
						String ednsStr;
						int edns;
	
						if (argv[arg].length() > 2) {
							ednsStr = argv[arg].substring(2);
						} else {
							ednsStr = argv[++arg];
						}
	
						edns = Integer.parseInt(ednsStr);
	
						if ((edns < 0) || (edns > 1)) {
							LOGGER.info("DNS: Unsupported " + "EDNS level: "
									+ edns);
	
							return null;
						}
	
						res.setEDNS(edns);
	
						break;
	
					case 'd':
						res.setEDNS(0, 0, ExtendedFlags.DO, null);
	
						break;
	
					case 'q':
						printQuery = true;
	
						break;
	
					default:
						LOGGER.info("DNS: Invalid option: ");
						LOGGER.info("DNS: " + argv[arg]);
					}
	
					arg++;
				}
			}
			
			if (name == null) {
				return null;
			}
			
			rec = Record.newRecord(name, type, dclass);
			query = Message.newQuery(rec);

			if (printQuery) {
				System.out.println(query);
			}

			response = res.send(query);
			
		} catch (ArrayIndexOutOfBoundsException e) {
			LOGGER.info("DNS: " + e.toString());
			return null;
		} catch (TextParseException e) {
			LOGGER.info("DNS: " + e.toString());
			return null;
		} catch (IOException e) {
			
			if (e instanceof SocketTimeoutException) {
				
				if (DasuConfiguration.getInstance().getDnsTimeout() != 0) {
					
					 launchDnsTimeout(server);
				}
			}
			
			LOGGER.info("DNS: " + e.toString());
			return null;
		}

		return response;		
	}
	
	public ArrayList<ArrayList<String>> getMyDnsServers() {		
		return getMyDns(Main.getPublicIpAddress());
	}
	
	public ArrayList<ArrayList<String>> getMyDns(String myIp) {
		
		if (myDns != null)
			return myDns;
		
		String publicDnsIp, publicDnsIpAna;
		myDns = new ArrayList<ArrayList<String>>();	
		org.xbill.DNS.ResolverConfig DNSConfig = new org.xbill.DNS.ResolverConfig();
		int order = 0;
		String type = "";
	
		for (String n : DNSConfig.servers()) {

			order++;
			publicDnsIp = null;
			publicDnsIpAna = null;
			
			if (DasuConfiguration.getInstance().isDoAnaLookup()) {
				 publicDnsIpAna = getPublicDnsIpAna(n, myIp);
				 type = "A";
			}
			else {
				publicDnsIp = getMyPublicDnsIp( n, myIp );
				type = "P";
			}

			if (DEBUG)
				LOGGER.info("DNS: " + n + " " + publicDnsIpAna + " " + order);
			
			if ( (!publicDnsIpAna.equals(n) || (publicDnsIpAna.equals(n) && (!isPrivateIp(n)))) ) {
				
				ReportableFact rf = new ReportableFact("DnsMapping");
				rf.addContentsObject("myIp", myIp);
				rf.addContentsObject("publicDnsIp", publicDnsIpAna);
				rf.addContentsObject("privateDnsIp", n);
				rf.addContentsObject("dnsType", "A");
				rf.addContentsObject("order", order);
				Statistics.getInstance().addReportableFact(rf);
				
			}

			else {
				
				if ( isPrivateIp(n)) {
				  
					ReportableFact rf = new ReportableFact("DnsMapping");
					rf.addContentsObject("myIp", myIp);
					rf.addContentsObject("publicDnsIp", publicDnsIp);
					rf.addContentsObject("privateDnsIp", n);
					rf.addContentsObject("dnsType", "P");
					rf.addContentsObject("order", order);
					Statistics.getInstance().addReportableFact(rf);
			  }
			  		
			}
			
			ArrayList<String> in = new ArrayList<String>();
			in.add(n);
			in.add(type.equals("A") ? publicDnsIpAna : publicDnsIp);
			in.add(type);
			myDns.add(in);			
						
		}
		
		return myDns;
	}
	
	public String getPublicDnsIpAna (String dnsIp, String myIp) {
	
		String returnDnsIp = dnsIp;
		Message msg = doLookup(myIp+".ana-aqualab.cs.northwestern.edu", dnsIp, null, -1);
		
		if (DEBUG)
			LOGGER.info("DNS: " + msg);
		
		if ( msg != null ) {
			Record[] records = msg.getSectionArray(Section.ANSWER);
			if (records.length != 0) {				
				returnDnsIp = records[0].rdataToString();	
			}
		}
		
		return returnDnsIp; 
	}
	
	public String getMyPublicDnsIp (String dnsIp, String myIp) {

		String returnDnsIp = dnsIp;
		
		Message msg = doLookup(myIp, null, "-x", -1);
		if ( msg != null ) {
			
			Record[] records = msg.getSectionArray(Section.ANSWER);
			if (records!=null && records.length != 0) {
				String ipAddressResponse = records[0].rdataToString();
				ipAddressResponse = 
					ipAddressResponse.substring(ipAddressResponse.indexOf("."));
				msg = doLookup(ipAddressResponse.substring(1), null, "-t NS", -1);
				records = msg.getSectionArray(Section.ANSWER);

				if (records.length != 0) {
					returnDnsIp = records[0].rdataToString();
					
					msg = doLookup(records[0].rdataToString(), null, null, -1);
					records = msg.getSectionArray(Section.ANSWER);
					
					if (records.length != 0) {
						returnDnsIp = records[0].rdataToString();
					}
				}
			}
		}
		
		if (!isValidIp(returnDnsIp))
			return dnsIp;
		
		return returnDnsIp;
	}
	
	private boolean isPrivateIp(String ip) {

		 if (ip.startsWith("127.") || ip.startsWith("192.168") || ip.startsWith("10.") || 
				 ip.matches("172.1[6-9].[\\d]{1,3}\\.[\\d]{1,3}") || ip.matches("172.2[0-9].[\\d]{1,3}\\.[\\d]{1,3}") ||
				 ip.matches("172.3[0-1].[\\d]{1,3}\\.[\\d]{1,3}"))
 			 return true;
		 
		 return false;  

	 }   
	
	public void launchDnsTimeout(final String dnsIp)
	{
	  
		if (DEBUG)
			LOGGER.info("DEBUG: DasuDnsModule: Timeout!!!" + timeoutIsRunning);
		
	  if (!timeoutIsRunning.containsKey(dnsIp)) {

		timeoutIsRunning.put(dnsIp, Util.currentGMTTime());
		
	  	Main.createThread("TimeoutDigger-" + dnsIp, new Runnable(){

			public void run() {
				org.xbill.DNS.Name name = null;
				int type = org.xbill.DNS.Type.A;
				int dclass = DClass.IN;
				boolean again = true;
				boolean giveUp = false;

				Message query;
				Record rec;
				SimpleResolver res = null;
				int timeout = DasuConfiguration.getInstance().getDnsTimeout();
				
				long b4 = Util.currentGMTTime();
				int i = 1;
				while (again) {
				  try {
					name = Name.fromString("google.com", Name.root);
					res = new SimpleResolver(dnsIp);
					rec = Record.newRecord(name, type, dclass);
					query = Message.newQuery(rec);
					res.send(query);
					again = false;
				  
				  } catch (IOException e1) {
					  
					  if (DEBUG)
						  LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " dns TIMEOUT: try again in " + timeout + " seconds...");
					  
					  if (Util.currentGMTTime() - b4 >= i*5*60*1000) {
						  timeout *= 2;
						  i++;
						  
						  if (DEBUG)
							  LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " timeout: " + timeout);
					  }
						  
					  //try again until timeout >= 10 mins, then give up...
					  if ( timeout >= 10*60 ) {
						  
						  if (DEBUG)
							  LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " dns TIMEOUT: giving up...");
						  
						  again = false;
						  giveUp = true;
					  }
					  else {
				  
						  try {
								Thread.sleep(timeout*1000);
							  } catch (InterruptedException e) {
								//e.printStackTrace();
							  }
							  again = true;	
					  }
				  }
				  
				} //while
				
				long after = Util.currentGMTTime();
				if (DEBUG) {
					LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " dns TIMEOUT: back online...");
					LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + ": " + Long.toString(after - b4));
				}
				
				timeoutIsRunning.remove(dnsIp);		
				
				FactDnsTimeout ftimeout =  new FactDnsTimeout(dnsIp, b4, after, giveUp, dnsIp);
				reportProbeResult(ftimeout);
				AMCoordinatorV2.assertToEngine(SourceType.RE, ftimeout);
				
			} //run
		     	
	  	 });


	  }	// if

	}

	@Override
	public Integer getNumberActiveProbes() {
		
		if (!isIdle())
			return 1;
		
		return 0;
		
	}
	
	public static void main(String args[]) {
		
		Message msg = DasuDnsModule.getInstance().doLookup("l4.yimg.com", null, null, -1);
		FactDnsProbeResult ret = new FactDnsProbeResult("", "l4.yimg.com", null,
				Util.currentGMTTime(), msg, Util.currentGMTTime(), "type", -1);
		System.out.println(ret.getResponse().getAnswer());
		System.out.println(ret.getResponse().getQuestion());
		System.out.println(ret.getResponse().getAdditional());
		System.out.println(ret.getResponse().getAuthority());
		
		System.out.println(msg);
		
		List l = Arrays.asList(new String[]{"http://google.com", "http://facebook.com", "http://youtube.com", "http://yahoo.com", "http://baidu.com", "http://wikipedia.org", "http://qq.com", "http://linkedin.com", "http://live.com", "http://twitter.com", "http://amazon.com", "http://blogspot.com", "http://taobao.com", "http://google.co.in", "http://wordpress.com", "http://bing.com", "http://yahoo.co.jp", "http://yandex.ru", "http://sina.com.cn", "http://ebay.com", "http://google.de", "http://vk.com", "http://tumblr.com", "http://google.co.uk", "http://googleusercontent.com", "http://pinterest.com", "http://msn.com", "http://google.fr", "http://google.com.br", "http://google.co.jp", "http://mail.ru", "http://163.com", "http://hao123.com", "http://microsoft.com", "http://paypal.com", "http://google.com.hk", "http://google.ru", "http://ask.com", "http://xvideos.com", "http://blogger.com", "http://instagram.com", "http://craigslist.org", "http://imdb.com", "http://weibo.com", "http://apple.com", "http://google.it", "http://google.es", "http://tmall.com", "http://bbc.co.uk", "http://sohu.com", "http://xhamster.com", "http://t.co", "http://fc2.com", "http://amazon.co.jp", "http://stackoverflow.com", "http://google.com.mx", "http://soso.com", "http://cnn.com", "http://google.ca", "http://adobe.com", "http://conduit.com", "http://imgur.com", "http://go.com", "http://alibaba.com", "http://flickr.com", "http://wordpress.org", "http://akamaihd.net", "http://360.cn", "http://neobux.com", "http://odnoklassniki.ru", "http://delta-search.com", "http://ku6.com", "http://ifeng.com", "http://thepiratebay.sx", "http://blogspot.in", "http://vube.com", "http://huffingtonpost.com", "http://godaddy.com", "http://amazon.de", "http://adcash.com", "http://pornhub.com", "http://about.com", "http://google.com.tr", "http://bp.blogspot.com", "http://youku.com", "http://google.com.au", "http://ebay.de", "http://adf.ly", "http://uol.com.br", "http://google.pl", "http://reddit.com", "http://ebay.co.uk", "http://aol.com", "http://alipay.com", "http://dailymotion.com", "http://netflix.com", "http://cnet.com", "http://babylon.com", "http://espn.go.com", "http://livejasmin.com"});
		Collections.shuffle(l);
		System.out.println(l.get(0));
		

		
	}
	

}
