/**
 * Dasu Project
 *
 * File:         DasuYouTubeModule.java
 * RCS:          $Id$
 * Description:  DasuYouTubeModule class (see below)
 * Author:       Zachary Bischof
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jan 17, 2013 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2013, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.dynamic.measurement;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.*;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.dynamic.drools.FactYouTubeProbeResult;
import edu.northwestern.dasu.measurement.DasuProbeModule;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.measurement.types.ProbeType;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.measurement.types.TaskType;
import edu.northwestern.dasu.netinterface.NetInterfaceClassifier;
import edu.northwestern.dasu.netinterface.NetInterfaceClassifier.InterfaceType;
import edu.northwestern.dasu.util.Util;

/**
 * @author Zachary Bischof &lt;zbischof@u.northwestern.edu&gt;
 * 
 *         The DasuYouTubeModule class manages the set of YouTube tests performed
 *         by the peer.
 */

public class DasuYouTubeModule extends DasuProbeModule<FactYouTubeProbeResult> {

	private static DasuYouTubeModule self;	
	Shell shell = null;
	private String browserType = "";
	private static Browser browser;
	private static Boolean testFinished = false;
	private static String jsonResult = "";
	private static Integer loadLengthTime = 30; //180;//180; //seconds
	private static Integer sleepInterval = 10; //loadLengthTime/6; //seconds	
	private static Boolean testInterrupted = false;

	private String jsonLibrary = "var JSON = JSON || {};" +
			"JSON.stringify = JSON.stringify || function (obj) {" +
			"var t = typeof (obj);" +
			"if (t != \"object\" || obj === null) {" +
			"if (t == \"string\") obj = '\"'+obj+'\"';" +
			"return String(obj);" +
			"} else {" +
			"var n, v, json = [], arr = (obj && obj.constructor == Array);" +
			"for (n in obj) {" +
			"v = obj[n]; t = typeof(v);" +
			"if (t == \"string\") v = '\"'+v+'\"';" +
			"else if (t == \"object\" && v !== null) v = JSON.stringify(v);" +
			"json.push((arr ? \"\" : '\"' + n + '\":') + String(v));" +
			"}" +
			"return (arr ? \"[\" : \"{\") + String(json) + (arr ? \"]\" : \"}\");" +
			"}" +
			"};" +
			"JSON.parse = JSON.parse || function (str) {" +
			"if (str === \"\") str = '\"\"';" +
			"eval(\"var p=\" + str + \";\");" +
			"return p;" +
			"};";

	private String jsTestCode = "var allData = {};var testRuntime = 1000*" + loadLengthTime + ";var player;var videoQualities;var qualityIndex = 0;var videoBytesTotal;var testStartTime = -1;var testsInitialized = false;var currTestData = [];var state;var finished = false;function manageTests(){ if(testsInitialized == false){ initializeTests(); return; } if(testStartTime == -1){ var currentTime; var bytesLoaded; if(videoQualities.length > 1){  player.setPlaybackQuality(videoQualities[qualityIndex + 1 % videoQualities.length]);  player.seekTo(0,true);  player.setPlaybackQuality(videoQualities[qualityIndex]);  currentTime = new Date().getTime();  bytesLoaded = player.getVideoBytesLoaded(); } else {  currentTime = new Date().getTime();  bytesLoaded = player.getVideoBytesLoaded(); } var fraction = player.getVideoLoadedFraction(); testStartTime = currentTime; currTestData.push({'timestamp':currentTime, 'bytesLoaded': bytesLoaded, 'state' : state, 'stateChangeTrigger' : false, 'fractionLoaded' : fraction}); } else { var currentTime = new Date().getTime(); var bytesLoaded = player.getVideoBytesLoaded(); var fraction = player.getVideoLoadedFraction(); currTestData.push({'timestamp':currentTime, 'bytesLoaded': bytesLoaded, 'state' : state, 'stateChangeTrigger' : false, 'fractionLoaded' : fraction}); if(currentTime - testStartTime > testRuntime || fraction == 1){  var size = player.getVideoBytesTotal();  allData[videoQualities[qualityIndex]] = {'data' : currTestData, 'size' : size};  currTestData = [];  testStartTime = -1;  qualityIndex++;  if(qualityIndex >= videoQualities.length){  finished = true;  window.clearInterval(testInterval);  } } } player.mute();}function initializeTests(){ try{ player = document.getElementById('movie_player-flash'); player.mute(); videoQualities = player.getAvailableQualityLevels(); videoQualitiesRemaining = player.getAvailableQualityLevels(); player.addEventListener('onStateChange', 'stateChangeListener'); videoBytesTotal = player.getVideoBytesTotal(); testsInitialized = true; } catch (e) { return; }}function stateChangeListener(newState){ var currentTime = new Date().getTime(); var bytesLoaded = player.getVideoBytesLoaded(); state = newState; var fraction = player.getVideoLoadedFraction(); currTestData.push({'timestamp':currentTime, 'bytesLoaded': bytesLoaded, 'state' : state, 'stateChangeTrigger' : true, 'fractionLoaded': fraction});};var testInterval = window.setInterval(manageTests, 1000);";

	public DasuYouTubeModule() {
		super("DasuYouTubeModule", 1.0f);
	}
	
	public boolean isIdle() {
		return idle;
	}

	public synchronized static DasuYouTubeModule getInstance() {
		if (self == null) {
			self = new DasuYouTubeModule();
			self.setDaemon(true);
		}
		return self;
	}

	public void updateJstTestCode() {
		jsTestCode = "var allData = {};var testRuntime = 1000*" + loadLengthTime + ";var player;var videoQualities;var qualityIndex = 0;var videoBytesTotal;var testStartTime = -1;var testsInitialized = false;var currTestData = [];var state;var finished = false;function manageTests(){ if(testsInitialized == false){ initializeTests(); return; } if(testStartTime == -1){ var currentTime; var bytesLoaded; if(videoQualities.length > 1){  player.setPlaybackQuality(videoQualities[qualityIndex + 1 % videoQualities.length]);  player.seekTo(0,true);  player.setPlaybackQuality(videoQualities[qualityIndex]);  currentTime = new Date().getTime();  bytesLoaded = player.getVideoBytesLoaded(); } else {  currentTime = new Date().getTime();  bytesLoaded = player.getVideoBytesLoaded(); } var fraction = player.getVideoLoadedFraction(); testStartTime = currentTime; currTestData.push({'timestamp':currentTime, 'bytesLoaded': bytesLoaded, 'state' : state, 'stateChangeTrigger' : false, 'fractionLoaded' : fraction}); } else { var currentTime = new Date().getTime(); var bytesLoaded = player.getVideoBytesLoaded(); var fraction = player.getVideoLoadedFraction(); currTestData.push({'timestamp':currentTime, 'bytesLoaded': bytesLoaded, 'state' : state, 'stateChangeTrigger' : false, 'fractionLoaded' : fraction}); if(currentTime - testStartTime > testRuntime || fraction == 1){  var size = player.getVideoBytesTotal();  allData[videoQualities[qualityIndex]] = {'data' : currTestData, 'size' : size};  currTestData = [];  testStartTime = -1;  qualityIndex++;  if(qualityIndex >= videoQualities.length){  finished = true;  window.clearInterval(testInterval);  } } } player.mute();}function initializeTests(){ try{ player = document.getElementById('movie_player-flash'); player.mute(); videoQualities = player.getAvailableQualityLevels(); videoQualitiesRemaining = player.getAvailableQualityLevels(); player.addEventListener('onStateChange', 'stateChangeListener'); videoBytesTotal = player.getVideoBytesTotal(); testsInitialized = true; } catch (e) { return; }}function stateChangeListener(newState){ var currentTime = new Date().getTime(); var bytesLoaded = player.getVideoBytesLoaded(); state = newState; var fraction = player.getVideoLoadedFraction(); currTestData.push({'timestamp':currentTime, 'bytesLoaded': bytesLoaded, 'state' : state, 'stateChangeTrigger' : true, 'fractionLoaded': fraction});};var testInterval = window.setInterval(manageTests, 1000);";
	}

	@Override
	protected FactYouTubeProbeResult probeIp(ProbeSubTask pst) {

		if(pst.getParams() != null && pst.getParams().containsKey("loadLengthTime")) {		
			loadLengthTime = (Integer) pst.getParams().get("loadLengthTime");
			updateJstTestCode();			
		}

		long timestamp = Util.currentGMTTime();

		LOGGER.info("YOUTUBE_MODULE: Loading " + pst.getIpAddress());
		String result = browse(pst.getIpAddress());

		String interfaceName = Main.getInterfaceName();		
		NetInterfaceClassifier interfaceClassifier = new NetInterfaceClassifier(interfaceName);
		InterfaceType interfaceType = interfaceClassifier.getInterfaceType();

		// Record results
		FactYouTubeProbeResult ret = new FactYouTubeProbeResult(pst.getIpAddress(), -1,
				timestamp, interfaceName, interfaceType.toString(), browserType, result);

		ret.setInterrupted(testInterrupted);
		ret.setPst(pst);
		return ret;

	}

	private static Long startTime = System.currentTimeMillis();
	private volatile static Long endTime = startTime + 45*1000;

	private String browse(final String url) {

		synchronized(testFinished) {
			testFinished = false;
		}
		synchronized(testInterrupted) {
			testInterrupted = false;
		}

		synchronized(jsonResult) {
			jsonResult = "";
		}
		new Thread(new Runnable(){
			public void run(){

				Display.getDefault().asyncExec(new Runnable() {
					public void run() {

						if (shell != null) {
							shell.dispose();
						}

						shell = new Shell(Display.getDefault());
						shell.setText("YouTube Browser");
						shell.setSize(100, 100);
						shell.setVisible(false);						
						browser = new Browser(shell, SWT.NONE);
						shell.setLayout(new FillLayout());
						browser.setJavascriptEnabled(true);
						browserType = browser.getBrowserType();

						browser.addProgressListener(new ProgressListener() {
							public void changed(ProgressEvent event) {
							}

							public void completed(ProgressEvent event) {
								browser.execute("var allData = {};");
								browser.execute("function getFinished(){return finished;}");
								browser.execute("function getAllData(){return JSON.stringify(allData);}");
								browser.execute("function getVideoQualities(){return videoQualities.length;};");
								browser.execute(jsonLibrary);
								browser.execute("testRuntime = " + loadLengthTime*1000);
								if(url.contains("aqua-lab.org/dasu/video_test/")) {
									browser.execute("startTest();");
								} else if (url.contains("www.youtube.com/my_speed")){									
									browser.execute("allData['youtube_test'] = document.getElementById('content').innerHTML;");
									synchronized(jsonResult) {
										jsonResult = (String) browser.evaluate("return getAllData();");
									}
									synchronized(testFinished){
										testFinished = true;
									}
								}
							}

						});
						browser.setUrl(url);						
					}

				});

				startTime = System.currentTimeMillis();
				endTime = startTime + 45*1000;

				StringBuilder sb = new StringBuilder();
				sb.append("YOUTUBE_MODULE: before launching async thread!\n");
				sb.append("YOUTUBE_MODULE: loadLengthTime: " + loadLengthTime + "\n");				
				sb.append("YOUTUBE_MODULE: sleepInterval: " + sleepInterval + "\n");
				sb.append("YOUTUBE_MODULE: startTime: " + new java.util.Date(startTime) + "\n");									
				sb.append("YOUTUBE_MODULE: currentTime: " + new java.util.Date(System.currentTimeMillis()) + "\n");
				sb.append("YOUTUBE_MODULE: endTime: " + new java.util.Date(endTime) + "\n");				
				sb.append("YOUTUBE_MODULE: testFinished: " + testFinished + "\n");
				LOGGER.info(sb.toString());


				while( System.currentTimeMillis() < endTime && !testFinished) {

					if(url.contains("http://aqua-lab.org/dasu/video_test/")){
						
						Display.getDefault().asyncExec(new Runnable() {
							public void run() {
								try{

									Boolean testFinishedRunning = (Boolean) browser.evaluate("return getFinished();");
									LOGGER.info("YOUTUBE_MODULE: checking testFinished: " + testFinishedRunning);
									synchronized(jsonResult) {
										jsonResult = (String) browser.evaluate("return getAllData();");
									}
									browser.execute("startTest();");
									if(testFinishedRunning){
										//jsonResult = (String) browser.evaluate("return getAllData();");
										shell.dispose();
										synchronized(testFinished) {
											LOGGER.info("YOUTUBE_MODULE: finished loading all data!");
											testFinished = true;
											testFinished.notifyAll();
										}
									}

									Double numQualities = (Double) browser.evaluate("return getVideoQualities();");								
									if ( numQualities != null) {

										StringBuilder sb = new StringBuilder();
										sb.append("YOUTUBE_MODULE: loadLengthTime: " + loadLengthTime + "\n");
										sb.append("YOUTUBE_MODULE: numQualities: " + numQualities + "\n");
										sb.append("YOUTUBE_MODULE: sleepInterval: " + sleepInterval + "\n");
										sb.append("YOUTUBE_MODULE: startTime: " + new java.util.Date(startTime) + "\n");									
										sb.append("YOUTUBE_MODULE: currentTime: " + new java.util.Date(System.currentTimeMillis()) + "\n");

										if ( startTime + numQualities*loadLengthTime*1000 + 45*1000 > endTime) {
											endTime = startTime + numQualities.intValue()*loadLengthTime*1000 + 45*1000;
										}

										sb.append("YOUTUBE_MODULE: endTime: " + new java.util.Date(endTime));
										LOGGER.info(sb.toString());
										//(Double)browser.evaluate("return testRuntime;");										
									}


								} catch (Exception e){
								}
							}
						});

						//sleep and try again
						synchronized(testFinished) {
							try {
								testFinished.wait(sleepInterval*1000);							
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
						}
					}
				}

				synchronized(testFinished) {	

					if (!testFinished) {
						sb = new StringBuilder();
						sb.append("YOUTUBE_MODULE: quiting... took too long\n" );
						sb.append("YOUTUBE_MODULE: startTime: " + new java.util.Date(startTime)+"\n");
						sb.append("YOUTUBE_MODULE: endtime: " + new java.util.Date(endTime)+"\n");							
						sb.append("YOUTUBE_MODULE: currTime: " + new java.util.Date(System.currentTimeMillis())+"\n");

						LOGGER.info("YOUTUBE_MODULE: " + sb.toString());

						synchronized(testInterrupted) {
							testInterrupted = true;						
						}						

						testFinished = true;						
					}
				}

				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						shell.dispose();
					}
				});

			}
		}).run();

		synchronized(testFinished) {
			while(!testFinished) {
				try {
					testFinished.wait(60*1000);					
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
		LOGGER.info("YOUTUBE_MODULE: Finished. Results: " + jsonResult);
		return jsonResult;
	}

//	public Pair<String, FactTemplate> proccessResult(long eventId) {
//		FactYouTubeProbeResult youTubeResult = resultTable.remove(eventId);
//		return new Pair<String, FactTemplate>(youTubeResult.url, youTubeResult);
//
//	}

	public HashMap<String, Object> dumpObj(Object obj) {
		HashMap<String, Object> ret = new HashMap<String, Object>();

		Method[] methods = obj.getClass().getMethods();
		for(int i = 0; i < methods.length; i++) {
			Method m = methods[i];
			String name = m.getName();

			String field = null;

			if(name.startsWith("get")) {
				field = name.substring(3, 4).toLowerCase() + name.substring(4);
			} else if(name.startsWith("is")) {
				field = name.substring(2, 3).toLowerCase() + name.substring(3);
			}

			if(field == null) continue;
			if(field.equals("class")) continue;
			if(field.equals("field")) continue;

			try {
				ret.put(field, m.invoke(obj, new Object[0]));
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
			}
		}

		return ret;
	}

	public static void main(String[] args) {


		DasuYouTubeModule.getInstance().start();

		Display display = new Display();
		Shell shell = new Shell(display);

		int taskId = -1;
		int priority = 0;
		long startTime = -1;
		long endTime = -1;
		HashMap params = new HashMap();
		params.put("loadLengthTime",10);
		//List<String> urlList = new ArrayList<String>(Arrays.asList(new String[]{"http://www.youtube.com/my_speed"}));
		List<String> urlList = new ArrayList<String>(Arrays.asList(new String[]{"http://aqua-lab.org/dasu/video_test/video_2m1s.html"}));
		ProbeTask task = new ProbeTask(taskId, priority, startTime, endTime, urlList, 
				ProbeType.valueOf("YOUTUBE"),TaskType.BULK, SourceType.RE, params);        		
		Iterator<ProbeSubTask> iter = task.getSubTasks().iterator();
		ProbeSubTask pst = iter.next();
		DasuYouTubeModule.getInstance().probe(pst);

		while (!shell.isDisposed ()) {
			if (display.readAndDispatch ()) {
				display.sleep();
			}
		}

	}
	
	@Override
	public Integer getNumberActiveProbes() {
		
		if (!isIdle())
			return 1;
		
		return 0;
		
	}


}
