/**
 * Dasu Project
 *
 * File:         SchedulerAggregator.java
 * RCS:          $Id: SchedulerAggregator.java,v 1.0 Feb 9, 2013 6:20:19 PM mas939 Exp $
 * Description:  SchedulerAggregator class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 9, 2013 at 6:20:19 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement.scheduler
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2013, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.scheduler;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.measurement.types.SourceType;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SchedulerAggregator class ...
 */
public class SchedulerAggregator implements SchedulerInterface<ProbeSubTask> {
	
	protected final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	
	@SuppressWarnings("rawtypes")
	private Map<SourceType, SchedulerInterface> scheds = new HashMap<SourceType, SchedulerInterface>();
	
	@SuppressWarnings("unchecked")
	public boolean schedulable(ProbeSubTask pst) {
		
		SourceType st = pst.getSourceType();
		LOGGER.info("SCHEDULER: schedulable() " + st + " available schedulers: " + scheds.keySet());
		if ( scheds.containsKey(st) ) {
			return scheds.get(st).schedulable(pst);
		}
		
		return false;		
		
	}

	@SuppressWarnings("rawtypes")
	public void addScheduler(SourceType st, SchedulerInterface sched) {
		scheds.put(st, sched);
	}
	
	
	@Override
	public String toString() {
		return scheds.toString();
	}
	
	
}
