package edu.northwestern.dasu.measurement.tasks;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Random;

import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.measurement.types.TestType;
import edu.northwestern.dasu.util.Jsonizable;

public class TestTask extends Jsonizable implements DasuTask<TestTask.TestSubTask>, Serializable {

	private static final long serialVersionUID = 1L;
	
	public LinkedList<Object> params = null;
	public TestType testType = null;
	public SourceType sourceType = SourceType.RE; /* to redirect output for task by the AMC*/
	private int priority = 0; 	/* by default all tests have the same priority */
		
	public TestTask(TestType testType, SourceType sourceType) {
		this.testType = testType;
		this.sourceType = sourceType;
	}
	
	public TestTask(TestType testType, Object[] params) {
		this.params = new LinkedList<Object>(Arrays.asList(params));
		this.testType = testType;
	}
	
	public TestTask(TestType testType, Object[] params, SourceType sourceType) {

		this.testType = testType;
		this.sourceType = sourceType;
		if ( params != null)
			this.params = new LinkedList<Object>(Arrays.asList(params));
	}
	
	public void setPriority(int prior) {
		priority = prior;
	}
	
	public Integer getPriority() {
		return priority;
	}
	
	
	public String toString() {
		return toJSON().toJSONString();
	}

	public TestType getType() {
		return testType;
	}
	
	public Collection<TestSubTask> getSubTaskList() {
		return Arrays.asList(new TestSubTask[]{new TestSubTask(this)});
	}
	
	public class TestSubTask extends Jsonizable implements DasuSubTask<TestType>, Serializable {

		private static final long serialVersionUID = 3L;
		
		private TestTask parent;
		private long id = System.nanoTime() + new Random().nextLong() + new Random(1000).nextLong();	
		public Map<String,Object> executionPathTimestamps = new HashMap<String, Object>();		
		
		public TestSubTask(TestTask parent) {
			this.parent = parent;
		}
		
		public TestType getType() { 
			return parent.getType();
		}
		
		public SourceType getSourceType() {
			return parent.sourceType;
		}

		public void recordExecutionPathTimestamp(String name, Object time) {
			executionPathTimestamps.put(name, time);
		}
		
		public Map<String,Object> getExecutionPathTimestamps() {
			return executionPathTimestamps;
		}
		
		public TestTask getParent() {
			return parent;
		}
		
		public Integer getPriority() {
			return parent.getPriority();
		}
		
		public String toString() {
			return toJSON().toJSONString();
		}

		public long getId() {			
			return id;
		}
		
	}

	public SourceType getSourceType() {
		return sourceType;
	}
		
}
