/**
 * Dasu Project
 *
 * File:         AzureusUpnpTest.java
 * RCS:          $Id$
 * Description:  AzureusUpnpTest class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:37:25 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.tests;

import java.net.InetAddress;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;
import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.internat.MessageText;
import org.gudy.azureus2.core3.ipchecker.natchecker.NatChecker;
import org.gudy.azureus2.core3.util.AEThread;
import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT;

import com.aelitis.azureus.core.AzureusCore;
import com.aelitis.azureus.core.AzureusCoreFactory;
import com.aelitis.azureus.core.AzureusCoreRunningListener;
import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;

import edu.northwestern.dasu.drools.FactUpnpTestResult;
import edu.northwestern.dasu.measurement.tasks.TestTask.TestSubTask;
import edu.northwestern.dasu.measurement.types.TestType;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The AzureusUpnpTest class ...
 *  The test is capable of receiving the port and protocol to test, 
 * this version takes default port and default tcp protocol 
 */
public class DasuTestAzureusUpnp extends DasuTestModule<FactUpnpTestResult> {

	int serverTCPListenPort;
	boolean DEBUG = true;
	private final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	private static Set<Checker> currentRun = new HashSet<Checker>();
	private static int maxCurrentRuns = TestType.AZUREUS_UPNP.getMaxPerMin();

	public DasuTestAzureusUpnp(float version) {
		super(version);
	}
  
	public class Checker extends AEThread {

		private int TCPListenPort;
		private boolean bContinue;
		private final AzureusCore core;
		private TestSubTask testSubTask;
	
	    public Checker(AzureusCore core, int tcp_listen_port, TestSubTask tst) {
	    	
	        super("NAT Checker");
	  		this.core = core;
	  		this.TCPListenPort = tcp_listen_port;
	        this.bContinue = true;
	        this.testSubTask = tst;
	        
	    }
	
	    public void runSupport() {
	    	
	    	if (DEBUG)
	    		LOGGER.info("TESTMODULE: " + MessageText.getString("configureWizard.nat.testing") + " " + TCPListenPort + " ... ");
	    	
	    		boolean success = false;
	    		boolean error = true;
	    		
	    		try {
	    			
	    			AzureusCore core = AzureusCoreFactory.getSingleton();
	    			NetworkAdmin netadmin = NetworkAdmin.getSingleton();	    				    			
	    			InetAddress[] multihome = netadmin.getMultiHomedServiceBindAddresses(true);
	    			
	    			NatChecker checker = new NatChecker(core, multihome[0], TCPListenPort, false);
		          
	    			synchronized(currentRun) {
		        	  currentRun.remove(this);
	    			}
	    			switch (checker.getResult()) {
	    			case NatChecker.NAT_OK :
		        	  if (DEBUG) 
		        		  LOGGER.info("TESTMODULE: " + MessageText.getString("configureWizard.nat.ok") + "\n" + checker.getAdditionalInfo());
		        	  
		        	  success = true;
		        	  error = false;
		        	  break;
		        	  //updateUpnpTestResultQueue(TCPListenPort, true, testSubTask);
		        	  //break;
		        	  
	    			case NatChecker.NAT_KO :
		        	  if (DEBUG)
		        		  LOGGER.info("TESTMODULE: " + MessageText.getString("configureWizard.nat.ko") + " - " + checker.getAdditionalInfo()+".\n");
		        	  
		        	  success = false;
		        	  error = false;
		        	  //updateUpnpTestResultQueue(TCPListenPort, false, testSubTask);
		        	  break;
		        	  
	    			default :
		        	  if (DEBUG)
		        		  LOGGER.info("TESTMODULE: " + MessageText.getString("configureWizard.nat.unable") + ". \n(" + checker.getAdditionalInfo()+").\n");
		          
		        	  success = false;
		        	  error = true;
		          	  //updateUpnpTestResultQueue(TCPListenPort, false, true, testSubTask);
		          	  break;
	    			}
	    		}
	    		catch (Exception e) {
	    			LOGGER.info("TESTMODULE: " + e.getMessage());
	    			//e.printStackTrace();
	    		}
	    		finally {
	    			updateUpnpTestResultQueue(TCPListenPort, success, error, testSubTask);
	    		}
	    		
	    }
	    
	}
  	
	public void runTest(final int TCPListenPort, final TestSubTask tst) {

		serverTCPListenPort = COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
	   	int	old_tcp 	= COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
	   	int	old_udp 	= COConfigurationManager.getIntParameter( "UDP.Listen.Port" );
	   	int	old_udp2 	= COConfigurationManager.getIntParameter( "UDP.NonData.Listen.Port" );
	    	
	    COConfigurationManager.setParameter("TCP.Listen.Port",serverTCPListenPort);
	        
	    if ( old_tcp == old_udp ){
	    	COConfigurationManager.setParameter("UDP.Listen.Port",serverTCPListenPort);
	    }
	    if ( old_tcp == old_udp2 ){
	    	COConfigurationManager.setParameter("UDP.NonData.Listen.Port",serverTCPListenPort);
	    }
	
	    COConfigurationManager.save();
	    
	    CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
			
			public void azureusCoreRunning(AzureusCore core) {		
				Checker checker = new Checker(core, TCPListenPort, tst);
				checker.start();
				synchronized(currentRun) {
					currentRun.add(checker);
				}
			}
		});
    
	}

	protected void updateUpnpTestResultQueue(int port, boolean success, boolean error, TestSubTask tst) {
		updateUpnpTestResultQueue(port, success, "tcp", error, tst);
	}
  
	protected void updateUpnpTestResultQueue(int port, boolean success, String protocol, boolean error, TestSubTask tst) {
	
		FactUpnpTestResult result = new FactUpnpTestResult(port, success, protocol, error);
		result.setTst(tst);
		resultTable.add(result);
		updateResultQueue(result, result.getTestType());
		currTestSubTask = null;
		
	}
	
	@Override
	public boolean isTestRunning() {
		
		boolean testRuning = false;
		synchronized(currentRun) {
			testRunning = currentRun.size() != 0;
		}
		
		return testRuning;
	}
	
	@Override
	public void runTest(TestSubTask tst) {
		
		int port = 12345;
		if ( tst.getParent().params != null ) {			
			port = (Integer) tst.getParent().params.pop();
		}
		
		/* return with error */
		if (currentRun.size() >= maxCurrentRuns || paused) {
			updateUpnpTestResultQueue(port, false, true, tst);
			return;
		}		

		runTest(port, tst);
		
	}
	



}
