/**
 * Dasu Project
 *
 * File:         DasuTestModule.java
 * RCS:          $Id: DasuTestModule.java,v 1.0 Feb 9, 2013 10:27:37 PM mas939 Exp $
 * Description:  DasuTestModule class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 9, 2013 at 10:27:37 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement.tests
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2013, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.tests;

import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.logging.Logger;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.drools.FactTestResult;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.tasks.TestTask.TestSubTask;
import edu.northwestern.dasu.measurement.types.EventType;
import edu.northwestern.dasu.stats.Statistics;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The DasuTestModule class ...
 */
public abstract class DasuTestModule<T extends FactTestResult> {	

	protected final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	private float version;
	protected Boolean testRunning = false;
	protected boolean paused = false;
	public ConcurrentLinkedQueue<T> resultTable = new ConcurrentLinkedQueue<T>();
	protected TestSubTask currTestSubTask = null;
	
	public DasuTestModule(float version) {
		this.version = version;	
	}
		
	public boolean isTestRunning() {
		return testRunning;
	}
	
	public void setPaused(boolean paused) {
		this.paused = paused;
	}
	
	public boolean isPaused() {
		return paused;
	}

	public abstract void runTest(TestSubTask tt);
	
	public float getVersion() {
		return version;
	}
	
	protected synchronized void updateResultQueue(T fact, EventType type) {
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(type);
			AMCoordinatorV2.events.notifyAll();
		}
		
		reportProbeResult(fact);
	}
	
	protected void reportProbeResult(FactTemplate fact) {
		
		if (Main.DEBUG)
			LOGGER.info("TESTMODULE: reportTestResult: " + fact);
		
		Statistics.getInstance().addReportableFact(fact);
	}	
	
	public T processResult() {
		return resultTable.poll();		
	}

	
}