/**
 * Dasu Project
 *
 * File:         ProbeType.java
 * RCS:          $Id$
 * Description:  ProbeType class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 2, 2010 at 8:31:59 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.measurement.types;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The ProbeType class ...
 */

public enum ProbeType implements EventType {
	PING(5, 150, 2),
	TRACEROUTE(5, 250, 2),
	HTTP(4, 20, 4),
	HTTP2(4, 20, 4),
	NDT(1, 1, 10, true, true),
	NDT2(1, 1, 10, false, true),
	SCRAPE(10, 100, 1),
	DNS(20, 1000, 1),
	BROWSER(1, 50, 5),
	YOUTUBE(1, 1, 10),
	PHANTOMJS(4, 20, 4);
	
	private int maxProbesPerHour;
	private int maxProbesPerMin;
	private String type;
	private int consumedBW; //units for now
	private boolean executeInIsolation = false; //pause torrents
	private boolean pauseOtherModule = false; //pause probes
	
	private ProbeType(int probesPerMinute, int probesPerHour, int consumedBW) {		
		this.maxProbesPerMin = probesPerMinute;
		this.maxProbesPerHour = probesPerHour;
		this.consumedBW = consumedBW;
		this.type = this.name();
	}

	private ProbeType(int probesPerMinute, int probesPerHour, int consumedBW, boolean pauseOtherModule, boolean executeInIsolation) {
		this(probesPerMinute,probesPerHour,consumedBW);
		this.pauseOtherModule = pauseOtherModule;
		this.executeInIsolation = executeInIsolation;
	}
	
	public int getMaxPerHour() {
		return maxProbesPerHour;
	}
	
	public int getMaxPerMin() {
		return maxProbesPerMin;
	}

	public int getConsumedBW() {
		return consumedBW;
	}
	
	public String getEventType() {
		return type;
	}
	
	public boolean isExecuteInIsolation() {
		return executeInIsolation;
	}
	
	public boolean isPauseOtherModules() {
		return pauseOtherModule;
	}
	
}