/**
 * Dasu Project
 *
 * File:         IntervalType.java
 * RCS:          $Id$
 * Description:  IntervalType class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 2, 2010 at 8:31:59 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.measurement.types;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The TestType class ...
 */

public enum TestType implements EventType {
	AZUREUS_UPNP(10,100,1), 
	AZUREUS_UL(1,1,10, true, false), 
	AZUREUS_DL(1,1,10, true, false), 
	AZUREUS_UL_ENCRYPTION(1,1,10, true, false), 
	AZUREUS_DL_ENCRYPTION(1,1,10, true, false),
	FORCE_SEEDING_THROUGH_PORT(1,1,10);
	
	private int maxTestsPerHour;
	private int maxTestsPerMin;
	private int consumedBW; //units for now
	private String type;
	private boolean executeInIsolation = false;
	private boolean pauseOtherModules = false;
	
	private TestType(int probesPerMinute, int probesPerHour, int consumedBW) {		
		this.maxTestsPerMin = probesPerMinute;
		this.maxTestsPerHour = probesPerHour;
		this.consumedBW = consumedBW;
		this.type = this.name();
	}
	
	private TestType(int probesPerMinute, int probesPerHour, int consumedBW, boolean pauseOtherModules, boolean executeInIsolation) {
		this(probesPerMinute, probesPerHour, consumedBW);
		this.executeInIsolation = executeInIsolation;
		this.pauseOtherModules = pauseOtherModules;
	}
	
	public int getMaxPerHour() {
		return maxTestsPerHour;
	}
	
	public int getMaxPerMin() {
		return maxTestsPerMin;
	}

	public String getEventType() {
		return type;
	}
	
	public int getConsumedBW() {
		return consumedBW;
	}
	
	public boolean isExecuteInIsolation() {
		return executeInIsolation;
	}

	public boolean isPauseOtherModules() {
		return pauseOtherModules;
	}
}