/**
 * Dasu Project
 *
 * File:         AntJVM.java
 * RCS:          $Id: AntJVM.java,v 1.0 Jul 21, 2012 11:25:50 AM mas939 Exp $
 * Description:  AntJVM class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 21, 2012 at 11:25:50 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.sandbox
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.sandbox;

import java.io.File;
import java.util.logging.Logger;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DefaultLogger;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Path;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.measurement.ame.AMEManager;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The AntJVM class ...
 */
public class AntJVM extends Thread {

	private static boolean DEBUG = false;
	protected final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	private Integer maxAMExperiments = 1;
	private String maxMemJVM = "64M";
	Integer retCode = -1;
	  
	
	public AntJVM( Integer maxNumAMExperiments, String maxMemJVM) {
		this.maxAMExperiments = maxNumAMExperiments;
		this.maxMemJVM = maxMemJVM;
		
		if ( Main.DEBUG && DasuConfiguration.getInstance().getDasuDebugFilter().contains("AME") ) {			
			DEBUG = true;
		}
		
	}
	
	private String getJarName(Float version) {
		String jarPrefix = DasuConfiguration.DASU_FILENAME_PREFIX;
		String jarSuffix = DasuConfiguration.DASU_FILENAME_SUFFIX;
		return jarPrefix + version.toString() + jarSuffix;
	}
	
	private String getJarPath(Float version) {		
		return Main.getDir() + File.separator + getJarName(version);
	}
	
	public void run() {

        Project project = new Project();
        project.setBaseDir(new File(System.getProperty("user.dir")));
        project.init();
        DefaultLogger logger = new DefaultLogger();
        project.addBuildListener(logger);

        logger.setOutputPrintStream(System.err);        
        logger.setErrorPrintStream(System.err);
        logger.setMessageOutputLevel(Project.MSG_ERR);
        
        if ( DEBUG ) {
        	logger.setMessageOutputLevel(Project.MSG_INFO);
        	//System.setOut(new PrintStream(new DemuxOutputStream(project, false)));
        	//System.setErr(new PrintStream(new DemuxOutputStream(project, true)));
        }
        
        project.fireBuildStarted();
        
        Throwable caught = null;
        try {

    		Java javaTask = new Java();
            javaTask.setTaskName("runjava");
            javaTask.setProject(project);
            javaTask.setArgs(maxAMExperiments.toString());
            javaTask.setFork(true);
            javaTask.setFailonerror(true);

            Path path = new Path(project, System.getProperty("java.class.path") + File.pathSeparator +
            		getJarPath(DasuConfiguration.getInstance().getInferenceVersion()) + File.pathSeparator + 
            		DasuManager.getInstance().getClass().getProtectionDomain().getCodeSource().getLocation().getPath());
            
			File f = new File(DasuManager.getInstance().getClass().getProtectionDomain().getCodeSource().getLocation().getFile());
			String dir = "\"" + f.toURI().toASCIIString().replace("file:", "file:///") + "\"";
			
            File e = new File(getJarPath(DasuConfiguration.getInstance().getInferenceVersion()));
            String externalJar = "\"" + e.toURI().toASCIIString().replace("file:", "file:///") + "\"";
            
            LOGGER.info("AME: dir: " + dir);
            LOGGER.info("AME: path: " + path);
            LOGGER.info("AME: externalJar: " + externalJar);
            
            javaTask.setClasspath(path);
            javaTask.setArgs(dir);                        
            javaTask.setArgs(externalJar);
            javaTask.setClassname(edu.northwestern.dasu.sandbox.mainSandboxedProcess.class.getName());
            javaTask.setJvmargs("-Xmx" + maxMemJVM);
            javaTask.init();
            retCode = javaTask.executeJava();

        } catch (BuildException e) {
                caught = e;
                e.printStackTrace();
        }
        
        project.log("finished");
        project.fireBuildFinished(caught);

        LOGGER.info("AME: finished... shutting down...");
        
        AMEManager.getInstance().stopAMEngines();
        
	}
	
	public void stopJVM() {
		LOGGER.info("STOP: stopping AntJVM");
		this.interrupt();
	}

}
