/**
 * Dasu Project
 *
 * File:         SandboxedResourceProfiler.java
 * RCS:          $Id: ResourceProfiler.java,v 1.0 Nov 16, 2011 10:59:51 AM mas939 Exp $
 * Description:  SandboxedResourceProfiler class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Nov 16, 2011 at 10:59:51 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.profilers
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.sandbox;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.profilers.ResourceProfiler;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SandboxedResourceProfiler class ... compute cpu usage on per-thread basis and memory usage.
 */

public class SandboxedResourceProfiler extends ResourceProfiler {
	

	public SandboxedResourceProfiler(long profileInterval, long limitUtilizationInterval, 
			float maxAllowedCpuUtilization, String[] threadNameFilterList) {
		super(profileInterval, limitUtilizationInterval, maxAllowedCpuUtilization, threadNameFilterList);
	}

	@Override
	protected void initiateShutdown(String reason) {
		mainSandboxedProcess.shutDown(reason);
	}
	
	@Override
	protected void commitReportableFact(ReportableFact rf) {
		mainSandboxedProcess.commitReportableFact(rf);
	}
	
	@Override
	protected boolean memoryExceededMaxAllowedUtiliation() {
		//no need to check for memory utilization, as JVM is instantiated with limited memory 
		return false;
	}

	public static void main(String[] args) throws InterruptedException {
		
		SandboxedResourceProfiler sp = new SandboxedResourceProfiler(
				DasuConfiguration.getInstance().getResourceProfilerRecordInterval(),
				DasuConfiguration.getInstance().getResourceProfilerLimitUtilizationInterval(),
				DasuConfiguration.getInstance().getMaxAllowedCpuUtilizationProfiler(),
				new String[]{""});
				
		Thread t = new Thread(sp);
		t.start();
		
		while ( t.isAlive() )
			Thread.sleep(1*1000);
	}
	
}
