package edu.northwestern.dasu.sandbox.applet;

import java.lang.reflect.Method;
import java.rmi.*;
import java.rmi.server.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Logger;

import edu.northwestern.dasu.drools.FactFireAction;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.drools.RuleBasedEngineInterface;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.timer.TimerEntry;

public class RmiAMCoordinatorImpl extends UnicastRemoteObject implements RmiAMCoordinatorInterface, RuleBasedEngineInterface {
    
	private static final long serialVersionUID = 1L;	
	private static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	private HashMap<SourceType, ArrayList<NotifyInterface>> notifyMapSet = null;
	private HashMap<SourceType, ArrayList<FactTemplate>> results = null;

	
	public RmiAMCoordinatorImpl() throws RemoteException { 
    	
    	  	notifyMapSet = new HashMap<SourceType, ArrayList<NotifyInterface>>();
    	  	results = new HashMap<SourceType, ArrayList<FactTemplate>>();
    }
        
	public Object syncMethodCall(String methodName, Object[] args) throws RemoteException {
		
		LOGGER.info("RMI: syncMethodCall(): " + methodName);
		for ( Method method : AMCoordinatorV2.class.getDeclaredMethods() ) {
			
			if (method.getName().equals(methodName) && method.getParameterTypes().length == args.length) {
				try {
					return method.invoke(AMCoordinatorV2.getInstance(), args);	
				} catch (Exception e) {
					LOGGER.warning("RMI: " + e.getMessage());
				}
			}
		}

		return null;
	}    
    
    public boolean nextAsyncTask(NotifyInterface notify) throws RemoteException {
    	    
    	LOGGER.info("RMI: nextAsyncTask(): " + notify.getEngineId() + " " + notify.getClass() + " " + notify.getParameter());
    	try {
    		
    		synchronized(notifyMapSet) {
    			if (!notifyMapSet.containsKey(notify.getEngineId())) {
    				notifyMapSet.put(notify.getEngineId(), new ArrayList<NotifyInterface>());    				
    			}
    			notifyMapSet.get(notify.getEngineId()).add(notify);
    			
    		}
    		
    		//if ( notify instanceof NotifyRemoteProbeTask)
    		if ( notify.getNotifyClassName().contains("NotifyRemoteProbeTask"))
    			return AMCoordinatorV2.addProbeTask(((ProbeTask)notify.getParameter()));
    		//else if ( notify instanceof NotifyRemoteTimerTask ) {
    		else if ( notify.getNotifyClassName().contains("NotifyRemoteTimerTask")) {
    			Object[] params = (Object[]) notify.getParameter();
    			String id = (String) params[0];
    			TimerEntry timerEntry = (TimerEntry) params[1];
    			FactFireAction fireAction = (FactFireAction) params[2];
    			AMCoordinatorV2.addTimerJob(id, timerEntry, fireAction);
    			return true;
    		}
	    	
    	}
    	catch (Exception e) {
    		e.printStackTrace();
    		return false;
    	}
    	
    	LOGGER.info("RMI: nextAsyncTask(): false \n\n\n\n\n\n");    	
    	return false;
    }
    
    public FactTemplate getResult(NotifyInterface notify) throws RemoteException {
    
    	LOGGER.info("RMI: getResult(): " + notify.getEngineId());
    	synchronized(results) {
    		
    		if ( results.containsKey(notify.getEngineId()))    			
    			return results.get(notify.getEngineId()).remove(0);
    	}
    	
    	return null;
    }
    
	public void addFact(FactTemplate factObject, SourceType engineId) {
		
		LOGGER.info("RMI: addFact(): " + engineId + " " + factObject);
		try {
			synchronized(results) {
				
				if ( !results.containsKey(engineId) )
					results.put(engineId, new ArrayList<FactTemplate>());
				
				results.get(engineId).add(factObject);
			}
			
			synchronized(notifyMapSet) {
				for ( SourceType st : notifyMapSet.keySet())
					LOGGER.info("RMI: addFact(): notifyMapSetSize: " + notifyMapSet.get(st).size());
				notifyMapSet.get(engineId).remove(0).doneIt();
			}
		} catch (RemoteException e) {
			//e.printStackTrace();
			LOGGER.info("AME: ERROR: " + e.getLocalizedMessage());
		}
	}

}