package edu.northwestern.dasu.sandbox.applet;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import edu.northwestern.cs.aqualab.cattle.jsonsimple.JSONObject;
import edu.northwestern.dasu.drools.FactFireAction;
import edu.northwestern.dasu.measurement.AMCoordinatorV2Interface;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.timer.TimerEntry;

public class RmiClientAMCoordinatorV2 implements AMCoordinatorV2Interface {
	
	private SourceType engineId;
	private RmiAMCoordinatorInterface remoteAMC;
	private final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	
    public RmiClientAMCoordinatorV2(SourceType engineId) {
        
        try {   
        	
        	remoteAMC = (RmiAMCoordinatorInterface) Naming.lookup("rmi://localhost/RmiAMCCoordinator");        	
        	this.engineId = engineId;
        }
        catch(Exception e){
        	e.printStackTrace();
        };        
    }

    public boolean addProbeTask(ProbeTask task) throws RemoteException {
    	
    	LOGGER.info("RMI: addProbeTask(): " + task);    	
    	task.sourceType = engineId;
    	for ( ProbeSubTask pst : task.getSubTasks()) {    		
    		
    		Map<String, Object> params = null;    		
    		if ( pst.getParams() != null ) {
    			params = new HashMap();
    			for ( Object key : pst.getParams().keySet() ) {
    				params.put(key.toString(), pst.getParams().get(key));
    			}
    		}
    		
            ProbeTask t = new ProbeTask(pst.getTaskId(), pst.getPriority(), pst.getWithinSeconds(),
            		Arrays.asList(new String[]{pst.toProbe}), task.probeType, task.taskType, pst.getSourceType(), params);
            
    		if ( task.target != null )
    			t.target = task.target;
    			
            NotifyRemoteProbeTask notifyProbeTask = new NotifyRemoteProbeTask(t,remoteAMC,engineId);
    		remoteAMC.nextAsyncTask(notifyProbeTask);
    	}
    	
    	return true;
    	
    }

	public ArrayList<ArrayList<String>> getMyDnsServers() throws RemoteException {
		LOGGER.info("RMI: getMyDnsServers()");
		return (ArrayList<ArrayList<String>>) remoteAMC.syncMethodCall("getMyDnsServers", null);		
	}

	public String getMyPublicIp() throws RemoteException {
		LOGGER.info("RMI: getMyPublicIp()");
		return (String) remoteAMC.syncMethodCall("getMyPublicIp", null);
	}

	public float getPluginVersion() throws Exception {
		LOGGER.info("RMI: getPluginVersion()");
		return (Float) remoteAMC.syncMethodCall("getPluginVersion", null);
	}

	public void commitFeedback(String type, String name, String email,
			String message, String dump) throws Exception {
		LOGGER.info("RMI: commitFeedback()");
		remoteAMC.syncMethodCall("commitFeedback", new Object[]{type, name, email, message, dump});
	}

	public void commitReportableFact(String type, Map<String, Object> values)
			throws Exception {
		
		LOGGER.info("RMI: commitReportableFact(): " + type);
		remoteAMC.syncMethodCall("commitReportableFact", new Object[]{type, values});
	}
	
	public JSONObject getJSONObjectFromServer(Integer taskId, String resource) 
			throws Exception {
		
		LOGGER.info("RMI: getJSONObjectFromServer(): " + taskId + " " + resource);
		return (JSONObject) remoteAMC.syncMethodCall("getJSONObjectFromServer", new Object[]{taskId, resource});
	}

	@SuppressWarnings("unchecked")
	public Map<String, Object> getEnvInfo() throws Exception {
		LOGGER.info("RMI: getEnvInfo()");
		return (Map<String, Object>) remoteAMC.syncMethodCall("getEnvInfo", new Object[]{});
	}

	public boolean addTimerJob(String id, TimerEntry timerEntry,
			FactFireAction fireAction) throws Exception {
		fireAction.setSourceType(engineId);
		
		LOGGER.info("RMI: addTimerJob(): " + timerEntry + " " + fireAction);
        NotifyRemoteTimerTask notifyTimerTask = new NotifyRemoteTimerTask(
        		new Object[]{id, timerEntry, fireAction},remoteAMC,engineId);
		return remoteAMC.nextAsyncTask(notifyTimerTask);

	}
	
	


}