/**
 * Dasu Project
 *
 * File:         StartActiveEngine.java
 * RCS:          $Id: StartActiveEngine.java,v 1.0 Jul 16, 2012 12:51:20 PM mas939 Exp $
 * Description:  StartActiveEngine class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 16, 2012 at 12:51:20 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.sandbox
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.sandbox;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.Field;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.server.RMISocketFactory;
import java.security.Policy;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.Map.Entry;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.drools.RuleBasedEngine;
import edu.northwestern.dasu.measurement.types.ProbeType;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.sandbox.applet.RemoteAMCPolicy;
import edu.northwestern.dasu.sandbox.applet.RmiAMEServerInterface;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SandboxedActiveEngine class ...
 */
public class mainSandboxedProcess {

	private final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	private static edu.northwestern.dasu.sandbox.AppletClassLoader classLoader = null;
	private static SandboxedResourceProfiler profiler = null;
	private static Thread profilerThread = null;
	private static HashMap<String, SandboxedActiveEngineClass> ActiveEngines = new HashMap<String, SandboxedActiveEngineClass>();
	private static HashMap<String, ArrayList<String>> ActiveEnginesRequest = new HashMap<String, ArrayList<String>>();
	private static RmiAMEServerInterface AMEServer = null;
	private static Integer maxAMExperiments = 1;
	private static String codesourceRmi = null;		
	public static Object notify = new Object();
	public static Boolean shutdown = false;
 	
	public static RuleBasedEngine getEngine(SourceType engineId) {
		
		System.out.println("\n getEngine: " + engineId + " " + ActiveEngines);
		if ( ActiveEngines.containsKey(engineId.toString() )) {
			return ActiveEngines.get(engineId.toString()).getAMEngine();
		}
		
		return null;
	}
	
	public static void initClassLoader() throws IOException {

		String path = System.getProperty("java.class.path");
        System.out.println("*" + path + "*");
        
		ArrayList<URL> urls = new ArrayList<URL>();			
		for ( String url : path.split(File.pathSeparator)) {
			
			try {
				System.out.println("*" + url + "*");
				urls.add(new URL("file:" + url));
				if ( classLoader == null )
					classLoader = new edu.northwestern.dasu.sandbox.AppletClassLoader(new URL("file:" + url),
							ClassLoader.getSystemClassLoader());
				else
					classLoader.addJar(url);
			}
			catch (Exception e) {
				System.out.println("*" + url + "*" + e.getLocalizedMessage());
			}
		}
		
	}
	
	public static void initializeAMEServer() {
		
		try {
			
			System.getProperties().put("java.rmi.server.useCodebaseOnly", "true");
			System.getProperties().put("java.rmi.server.codebase", codesourceRmi);
			System.getProperties().put("java.rmi.server.hostname", "127.0.0.1");
			System.out.println("java.rmi.server.hostname: " + System.getProperties().get("java.rmi.server.hostname"));
			System.out.println("java.rmi.server.codebase: " + System.getProperties().get("java.rmi.server.codebase"));			
			
			AMEServer = (RmiAMEServerInterface) Naming.lookup("rmi://localhost/RmiAMEServer");
			Policy.setPolicy(new RemoteAMCPolicy());
			System.setSecurityManager(new RMISecurityManager());
			
			LOGGER.info("AME: AMEServer: " + AMEServer);

			
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotBoundException e) {
			e.printStackTrace();
		}
		
	}
	
	public static void initializeProfiler() throws IOException {		
		
		profiler = 	new SandboxedResourceProfiler(				
				DasuConfiguration.getInstance().getResourceProfilerRecordInterval(), //1
				DasuConfiguration.getInstance().getResourceProfilerLimitUtilizationInterval(), //5
				DasuConfiguration.getInstance().getMaxAllowedCpuUtilizationProfiler(), //0.001f
				new String[]{""});

		profilerThread = new Thread(profiler);
		profilerThread.setName("DasuSandboxedResourceProfiler");
		profilerThread.start();
		
	}
	
	public static String getAvailableAME() {
		
		for ( SourceType st : SourceType.values()) {							
			if ( st.equals(SourceType.NULL) || st.equals(SourceType.RE) )
				continue;

			if ( !ActiveEngines.containsKey(st.toString()) && !ActiveEnginesRequest.containsKey(st.toString()) )
				return st.toString();
				
		}
		
		return null;
	}		
	
	public static void main(String[] args) {
				
		try {
					
			RMISocketFactory.setSocketFactory( new RMISocketFactory()
	        {
				
				//shutdown after two minutes
				int timeoutMillis = 120*1000;
	            public Socket createSocket( String host, int port ) throws IOException {
	                Socket socket = new Socket();
	                socket.setSoTimeout( timeoutMillis );
	                socket.setSoLinger( false, 0 );
	                socket.connect( new InetSocketAddress( host, port ), timeoutMillis );
	                return socket;
	            }

	            public ServerSocket createServerSocket( int port ) throws IOException {
	                return new ServerSocket( port );
	            }
	        } );
			
			maxAMExperiments = Integer.parseInt(args[0]);
			codesourceRmi = args[2] + " " + args[1];
			
			initClassLoader();
			addNewProbeTypes();
			initializeProfiler();
			initializeAMEServer();				
			
			//isActive = true;
			while (AMEServer.isServerAlive() && !isShutdown()) {
			
				String st = null;
				while ( ( st = getAvailableAME() ) != null && 
						( ActiveEnginesRequest.size() + ActiveEngines.size() < maxAMExperiments) ) {
					
					LOGGER.info("AME: Available SourceType: " + st);
					final String engineId = st.toString();
					Thread t = new Thread() {
													
						public void run() {					
							try {	
								
								ArrayList<String> ret = AMEServer.getAMExperiment(engineId);
								LOGGER.info("AME: " + ret);
								if ( ret != null ) {
									
									LOGGER.info("AME: Experiment received! " + ret);
									synchronized(ActiveEnginesRequest) {
										ActiveEnginesRequest.put(engineId, ret);
									}
									
								}
								else{

									synchronized(ActiveEnginesRequest) {
										ActiveEnginesRequest.remove(engineId);
									}

								}
								
								synchronized(notify) {
									notify.notify();
								}								

							} catch (RemoteException e) {
								e.printStackTrace();
							}
							
						}
					};
					
					ActiveEnginesRequest.put(st, null);
					t.start();
					
				}

				synchronized(notify) {
					notify.wait(25*1000);
				}

				Iterator<Entry<String, ArrayList<String>>> iter = ActiveEnginesRequest.entrySet().iterator();
				while ( iter.hasNext() ) {
					Entry<String, ArrayList<String>> entry = iter.next();
					if ( entry.getValue() != null ) {
						
						String engineId = entry.getKey();
						String[] arguments = entry.getValue().toArray(new String[entry.getValue().size()]);
						
						try {
							//remove policy constraints
							Policy.getPolicy().refresh();
							
							Class<?> c = null;			
							c = classLoader.loadClass("edu.northwestern.dasu.sandbox.applet.SandboxedActiveEngine");						
							SandboxedActiveEngineClass runnable = (SandboxedActiveEngineClass) 
									c.getConstructor(String[].class).newInstance(new Object[]{arguments}); 
							Thread t = new Thread(runnable);								
							ActiveEngines.put(engineId, runnable);
							
							//restart policy constraints
							Policy.getPolicy().refresh();
							
							t.start();
						}
						catch (Exception e) {

							LOGGER.info("AME: Downloaded experiment signature could not be verified or built, abort...");
							Writer writer = new StringWriter();
							PrintWriter printWriter = new PrintWriter(writer);
							e.printStackTrace(printWriter);
							String details = writer.toString();
							
							ReportableFact rf = new ReportableFact("ErrorEvent");
							rf.addContentsObject("report", "ExternalExperimentAborted");
							rf.addContentsObject("reason", "Downloaded experiment signature could not be verified or built, aborting..");
							rf.addContentsObject("stackTrace", details);
							rf.addContentsObject("arguments", Arrays.asList(arguments));						
							commitReportableFact(rf);
							
							//e.printStackTrace();
							AMEServer.AMECompleted(engineId, false);
						}
						
						iter.remove();
						
					}
				}	
				
				synchronized(notify) {
					notify.wait(5*1000);
				}

				boolean AMETaskFailed = false;
				
				// check if the main client wants to cancel the task (for instance because the
				// distributed coordinator has been inactive for too long)
				for ( String sourceType : ActiveEngines.keySet() ) {
					if ( AMEServer.isTaskCanceled(sourceType) ) {
						ActiveEngines.get(sourceType).getAMEngine().setActive(false);
						AMETaskFailed = true;
						LOGGER.info("AME: Task canceled by main client, abort...");
					}
				}
				
				Iterator<Entry<String, SandboxedActiveEngineClass>> iterAE = ActiveEngines.entrySet().iterator();
				while ( iterAE.hasNext() ) {
					Entry<String, SandboxedActiveEngineClass> entry = iterAE.next();
					if ( !entry.getValue().isAlive() ) {
						iterAE.remove();
						AMEServer.AMECompleted(entry.getKey(), !AMETaskFailed);
					}
					
				}

				LOGGER.info("AME: ActiveEngines: " + ActiveEngines);
				LOGGER.info("AME: ActiveEnginesRequest: " + ActiveEnginesRequest);

			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		LOGGER.info("AME: Server is not online, goodbye...");
		System.exit(0);
		
	}
	
	public static void shutDown(String reason) {
		
		ReportableFact rf = new ReportableFact("WatchdogTimer");
		rf.addContentsObject("jvmCpuUsage", profiler.getJvmCpuUsageList());
		rf.addContentsObject("dasuCpuUsageList", profiler.getDasuCpuUsageList());
		rf.addContentsObject("dasuMemoryUsageList", profiler.getMemoryUsage());
		rf.addContentsObject("report", "ExternalJvmShutdown");
		rf.addContentsObject("reason", reason);
		rf.addContentsObject("profileInterval", DasuConfiguration.getInstance().getResourceProfilerRecordInterval());
		rf.addContentsObject("limitUtilizationInterval", DasuConfiguration.getInstance().getResourceProfilerLimitUtilizationInterval());
	
		commitReportableFact(rf);
		shutdown = true;
		synchronized(notify) {
			notify.notify();
		}								

	}
	
	public static void commitReportableFact(ReportableFact rf) {
		
		try {
			AMEServer.commitReportableFact(rf);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}
	
	private static boolean isShutdown() {
		return shutdown;
	}
		
	private static void addNewProbeTypes() throws IOException {

		Properties map = new Properties();				
		map.load(classLoader.getResourceAsStream(
					DasuConfiguration.PROBE_MODULES_PROPERTIES));		
		LOGGER.info("AME: loading " + classLoader.getResource(DasuConfiguration.PROBE_MODULES_PROPERTIES));
		LOGGER.info("AME: loading types: " + map);
		
		try {
			
			//add new ProbeTypes if they don't already exit			
			for ( Object probeType : map.keySet()) {
				
				try {
					
					if ( !map.getProperty(probeType.toString()).contains("," ))
						continue;
					
					String maxVals[] = map.getProperty(probeType.toString()).split(":")[1].split(",");
					int maxProbesPerMin = Integer.parseInt(maxVals[0]);
					int maxProbesPerHour = Integer.parseInt(maxVals[1]);
					int consumedBW = Integer.parseInt(maxVals[2]);
					boolean pauseOtherModule = false;
					boolean executeInIsolation = false;
					
					if ( maxVals.length > 3 ) {
						pauseOtherModule = Boolean.parseBoolean(maxVals[3]);
						executeInIsolation = Boolean.parseBoolean(maxVals[4]);
					}
										
					if (!Util.probeTypeExists(probeType.toString())) {
						
						//doesnt exist, add it												 
						DynamicEnumFactory.addEnum(ProbeType.class, probeType.toString(), 
								new Class[]{int.class, int.class, int.class, boolean.class, boolean.class}, 
								new Object[]{maxProbesPerMin,maxProbesPerHour,consumedBW,pauseOtherModule,executeInIsolation});
						LOGGER.info("AME: adding a new probeType: "	+ probeType + " " 
								+ maxProbesPerMin + " " + maxProbesPerHour + " " + consumedBW);
						
					}
					else {
						
						HashMap<String, Object> toSet = new HashMap<String, Object>();
						toSet.put("maxProbesPerMin", maxProbesPerMin);
						toSet.put("maxProbesPerHour", maxProbesPerHour);
						toSet.put("consumedBW", consumedBW);
						toSet.put("pauseOtherModule", pauseOtherModule);
						toSet.put("executeInIsolation", executeInIsolation);
												
						try {
							
							LOGGER.info("AME: updating probeType: "	+ probeType + " " 
									+ maxProbesPerMin + " " + maxProbesPerHour + " " + consumedBW);
														
							for ( String name : toSet.keySet() ) {
								
								Field f = ProbeType.valueOf(probeType.toString()).getClass().getDeclaredField(name); 
								f.setAccessible(true);
								f.set(ProbeType.valueOf(probeType.toString()), toSet.get(name));
								f.setAccessible(false);
							}
							
						}
						catch (Exception e) {
							e.printStackTrace();
						}
					}
					
				}
				catch (Exception e) {
					//malformed new probeType, skip
					e.printStackTrace();
					LOGGER.warning("AME: malformed new probeType, skipping: "	+ probeType + " " +
							map.getProperty(probeType.toString()));

				}
				
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
	
}
