/**
 * Dasu Project
 *
 * File:         PlotObject.java
 * RCS:          $Id: PlotObject.java,v 1.0 Nov 14, 2011 3:55:13 PM mas939 Exp $
 * Description:  PlotObject class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Nov 14, 2011 at 3:55:13 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.simulator
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.simulator;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.ui.LinePlot;

/**
 * Dasu Project
 *
 * File:         PlotObject.java
 * RCS:          $Id: PlotObject.java,v 1.0 Nov 14, 2011 3:55:13 PM mas939 Exp $
 * Description:  PlotObject class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Nov 14, 2011 at 3:55:13 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.simulator
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The PlotObject class ...
 */
public class PlotObject {
	
	public String type = null;
	public String did = "";
	public HashMap<String, String> fieldsToLabels = new HashMap<String, String>();

	public PlotObject(String type) {
		this.type = type;
		
        AMCoordinatorV2.sessionSampleCreate(this.type + "_xaxis", 60);	        
        for ( long i=60; i>0; i--) {
            AMCoordinatorV2.sessionSampleSetValue(this.type + "_xaxis", i);
        } 

	}
	
	public PlotObject(String type, String did) {
		this.type = type;
		this.did = did;
		
        AMCoordinatorV2.sessionSampleCreate(this.type + this.did + "_xaxis", 60);	        
        for ( long i=60; i>0; i--) {
            AMCoordinatorV2.sessionSampleSetValue(this.type + this.did + "_xaxis", i);
        } 

	}

	
	public LinePlot getPlot(String fieldName) {
		
		if ( fieldsToLabels.keySet().contains(fieldName) ) {

			String plotName = type + "_" + fieldName;
			String plotTitle = fieldName;
							
			Collection<Float> yAxis = new ArrayList<Float>();
			Collection<Long> xAxis = new ArrayList<Long>();
			for ( Object obj : AMCoordinatorV2.sessionSampleGetValues(type + did + "_xaxis")) {
				xAxis.add(Long.parseLong(obj.toString()));
			}
			

			for ( Object obj : AMCoordinatorV2.sessionSampleGetValues(type + did + "_" + fieldName)) {
				yAxis.add(Float.parseFloat(obj.toString()));
			}

			LinePlot plot = new LinePlot(plotName, plotTitle, xAxis, yAxis, "(time)", fieldsToLabels.get(fieldName), LinePlot.FORMAT_NUMBER);
			return plot;
			
		}
		
		return null;
	}
	
	private void addField(String fieldName, String label) {			
		fieldsToLabels.put(fieldName, label);
	}

	public void addFields(Collection<String> fieldNames, Collection<String> labels) {
		
		if (fieldNames.size() != labels.size())
			return;
		
		Iterator<String> iterName = fieldNames.iterator();
		Iterator<String> iterLabel = labels.iterator();
		while (iterName.hasNext()) {
			String name = iterName.next();
			String label = iterLabel.next();
			addField(name, label);
		}

	}
	
	private boolean addTuple(Long x, Float y, String fieldName) {
		
		if ( fieldsToLabels.keySet().contains(fieldName) ) {
			
			String token = type + did + "_" + fieldName;
			if (!AMCoordinatorV2.sessionSampleExists(token)) 		
				AMCoordinatorV2.sessionSampleCreate(token, 60);
            AMCoordinatorV2.sessionSampleSetValue(token, y);
            AMCoordinatorV2.sessionSampleSetValue(type + did + "_xaxis", x);
			
			return true;
		}
		
		return false;
	}
	
	private boolean addTuple(Long x, Object y, String fieldName) {
		
		//System.out.println(fieldName + " " + y);
		if ( y == null)
			y = new Long(0);
		
		if ( fieldsToLabels.keySet().contains(fieldName) ) {
			
			Float yVal = Float.parseFloat(y.toString());				
			
			/**/
			String token = type + did + "_" + fieldName;
			if (!AMCoordinatorV2.sessionSampleExists(token)) 		
				AMCoordinatorV2.sessionSampleCreate(token, 60);
            AMCoordinatorV2.sessionSampleSetValue(token, yVal);
            AMCoordinatorV2.sessionSampleSetValue(type + did + "_xaxis", x);
			return true;				


		}
		
		return false;
	}
	
	public boolean addToPlot(FactTemplate factObject, Long time) {
		
		if ( !factObject.getClass().toString().contains(type) )
			return false;							
		
		try {
			
			BeanInfo beanInfoOfMyClass = Introspector.getBeanInfo(factObject.getClass());
			PropertyDescriptor propertiesOfMyClass[] = beanInfoOfMyClass.getPropertyDescriptors();

			for(int i=0;i<propertiesOfMyClass.length;i++){

				Method getter = propertiesOfMyClass[i].getReadMethod();
				Object value = getter.invoke(factObject, new Object[]{});
				String name = propertiesOfMyClass[i].getName();
				
				addTuple(time, value, name);					
			}
			
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		return true;
		
	}
	
	@Override
	public String toString() {
		
    	String ret = "PlotObject: <";
    	try {
        	
    		for ( Field f : this.getClass().getFields()) {
    		
    			if ( f.get(this) != null)
    				ret = ret.concat(f.getName() + ": " + f.get(this).toString() + " ");
    		}
    		
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    	
    	ret = ret.concat(">");
    	return ret;
		
	}

}