/**
 * Dasu Project
 *
 * File:         SimulatedDownload.java
 * RCS:          $Id: SimulatedDownload.java,v 1.0 Jul 26, 2011 10:02:36 AM mas939 Exp $
 * Description:  SimulatedDownload class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 26, 2011 at 10:02:36 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.simulator;

import java.io.File;
import java.net.URL;
import java.util.Map;

import org.gudy.azureus2.plugins.disk.DiskManager;
import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadActivationEvent;
import org.gudy.azureus2.plugins.download.DownloadActivationListener;
import org.gudy.azureus2.plugins.download.DownloadAnnounceResult;
import org.gudy.azureus2.plugins.download.DownloadAnnounceResultPeer;
import org.gudy.azureus2.plugins.download.DownloadAttributeListener;
import org.gudy.azureus2.plugins.download.DownloadCompletionListener;
import org.gudy.azureus2.plugins.download.DownloadException;
import org.gudy.azureus2.plugins.download.DownloadListener;
import org.gudy.azureus2.plugins.download.DownloadPeerListener;
import org.gudy.azureus2.plugins.download.DownloadPropertyListener;
import org.gudy.azureus2.plugins.download.DownloadRemovalVetoException;
import org.gudy.azureus2.plugins.download.DownloadScrapeResult;
import org.gudy.azureus2.plugins.download.DownloadStats;
import org.gudy.azureus2.plugins.download.DownloadStub;
import org.gudy.azureus2.plugins.download.DownloadTrackerListener;
import org.gudy.azureus2.plugins.download.DownloadWillBeRemovedListener;
import org.gudy.azureus2.plugins.download.savelocation.SaveLocationChange;
import org.gudy.azureus2.plugins.network.RateLimiter;
import org.gudy.azureus2.plugins.peers.Peer;
import org.gudy.azureus2.plugins.peers.PeerDescriptor;
import org.gudy.azureus2.plugins.peers.PeerManager;
import org.gudy.azureus2.plugins.peers.PeerManagerListener;
import org.gudy.azureus2.plugins.peers.PeerManagerListener2;
import org.gudy.azureus2.plugins.peers.PeerManagerStats;
import org.gudy.azureus2.plugins.peers.PeerReadRequest;
import org.gudy.azureus2.plugins.peers.PeerStats;
import org.gudy.azureus2.plugins.peers.Piece;
import org.gudy.azureus2.plugins.torrent.Torrent;
import org.gudy.azureus2.plugins.torrent.TorrentAnnounceURLList;
import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
import org.gudy.azureus2.plugins.torrent.TorrentEncodingException;
import org.gudy.azureus2.plugins.torrent.TorrentException;
import org.gudy.azureus2.plugins.torrent.TorrentFile;
import org.gudy.azureus2.plugins.utils.PooledByteBuffer;

import edu.northwestern.dasu.Main;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SimulatedDownload class ...
 */
public class SimulatedDownload implements Download {

	public String name = new String("dummy");
	public Long creationTime = new Long(0);
	public Integer downloadId = 0;
	public Integer priority = -1;
	public Integer position = -1;
	public Integer maximumDownloadKBPerSecond = -1;
	public Integer uploadRateLimitBytesPerSecond = -1;
	public long swarmPeerSpeed=-1;
	public int numPendingPeers=-1;
	public int state=-1;
	
	public SimulatedDownloadStats stats = new SimulatedDownloadStats();
	public SimulatedTorrent torrent = new SimulatedTorrent();
	public SimulatedDownloadAnnounceResult downloadAnnounceResult = new SimulatedDownloadAnnounceResult();
	public SimulatedDownloadScrapeResult downloadScrapeResult = new SimulatedDownloadScrapeResult();
	public SimulatedPeerManager peerManager = new SimulatedPeerManager();
	
	public SimulatedDownload(Integer downloadID) {
		this.name = this.name.concat(" " + downloadID.toString());
		this.downloadId = downloadID;
	}
	
	public SimulatedDownload(String name, Long creationTime, Integer downloadId) {
		this.name = name;
		this.creationTime = creationTime;
		this.downloadId = downloadId;
	}
	
	public int getDownloadId() {
		return downloadId;
	}
	
	@Override
	public boolean equals( Object obj) {
		
		if (obj instanceof SimulatedDownload)
			return this.downloadId.equals( ((SimulatedDownload) obj).downloadId );			
		
		return false;
	}
	
	@Override 
	public int hashCode() {
		return downloadId.hashCode();
	}
	
	public class SimulatedPeerManager implements PeerManager {

		public int numPeers = -1;
		
		public SimulatedPeerManagerStats peerManagerStats = new SimulatedPeerManagerStats();
		
		public class SimulatedPeerManagerStats implements PeerManagerStats {

			public int connectedSeeds =  -1;
			public int connectedLeechers = -1;			
		
			public int getConnectedLeechers() {
				return connectedLeechers;
			}

			public int getConnectedSeeds() {
				return connectedSeeds;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getDiscarded()
			 */
			public long getDiscarded() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getDownloadAverage()
			 */
			public long getDownloadAverage() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getDownloaded()
			 */
			public long getDownloaded() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getHashFailBytes()
			 */
			public long getHashFailBytes() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getPermittedBytesToReceive()
			 */
			public int getPermittedBytesToReceive() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getPermittedBytesToSend()
			 */
			public int getPermittedBytesToSend() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getUploadAverage()
			 */
			public long getUploadAverage() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#getUploaded()
			 */
			public long getUploaded() {
				// TODO Auto-generated method stub
				return 0;
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#permittedReceiveBytesUsed(int)
			 */
			public void permittedReceiveBytesUsed(int arg0) {
				// TODO Auto-generated method stub
				
			}

			/* (non-Javadoc)
			 * @see org.gudy.azureus2.plugins.peers.PeerManagerStats#permittedSendBytesUsed(int)
			 */
			public void permittedSendBytesUsed(int arg0) {
				// TODO Auto-generated method stub
				
			}
			
		}
		
		public Peer[] getPeers() {			
			return new Peer[numPeers];
		}
		
		public SimulatedPeerManagerStats getStats() {
			return peerManagerStats;		
		}
		
		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#addListener(org.gudy.azureus2.plugins.peers.PeerManagerListener)
		 */
		public void addListener(PeerManagerListener arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#addListener(org.gudy.azureus2.plugins.peers.PeerManagerListener2)
		 */
		public void addListener(PeerManagerListener2 arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#addPeer(org.gudy.azureus2.plugins.peers.Peer)
		 */
		public void addPeer(Peer arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#addPeer(java.lang.String, int)
		 */
		public void addPeer(String arg0, int arg1) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#addPeer(java.lang.String, int, boolean)
		 */
		public void addPeer(String arg0, int arg1, boolean arg2) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#addPeer(java.lang.String, int, int, boolean)
		 */
		public void addPeer(String arg0, int arg1, int arg2, boolean arg3) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#addPeer(java.lang.String, int, int, boolean, java.util.Map)
		 */
		public void addPeer(String arg0, int arg1, int arg2, boolean arg3,
				Map<Object, Object> arg4) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#createPeerStats(org.gudy.azureus2.plugins.peers.Peer)
		 */
		public PeerStats createPeerStats(Peer arg0) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getDiskManager()
		 */
		public DiskManager getDiskManager() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getDownload()
		 */
		public Download getDownload() throws DownloadException {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getDownloadRateLimitBytesPerSecond()
		 */
		public int getDownloadRateLimitBytesPerSecond() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getPeers(java.lang.String)
		 */
		public Peer[] getPeers(String arg0) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getPendingPeers()
		 */
		public PeerDescriptor[] getPendingPeers() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getPendingPeers(java.lang.String)
		 */
		public PeerDescriptor[] getPendingPeers(String arg0) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getPieces()
		 */
		public Piece[] getPieces() {
			// TODO Auto-generated method stub
			return null;
		}


		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#getUploadRateLimitBytesPerSecond()
		 */
		public int getUploadRateLimitBytesPerSecond() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#isSeeding()
		 */
		public boolean isSeeding() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#isSuperSeeding()
		 */
		public boolean isSuperSeeding() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#removeListener(org.gudy.azureus2.plugins.peers.PeerManagerListener)
		 */
		public void removeListener(PeerManagerListener arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#removeListener(org.gudy.azureus2.plugins.peers.PeerManagerListener2)
		 */
		public void removeListener(PeerManagerListener2 arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#removePeer(org.gudy.azureus2.plugins.peers.Peer)
		 */
		public void removePeer(Peer arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#requestCancelled(org.gudy.azureus2.plugins.peers.PeerReadRequest, org.gudy.azureus2.plugins.peers.Peer)
		 */
		public void requestCancelled(PeerReadRequest arg0, Peer arg1) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.peers.PeerManager#requestComplete(org.gudy.azureus2.plugins.peers.PeerReadRequest, org.gudy.azureus2.plugins.utils.PooledByteBuffer, org.gudy.azureus2.plugins.peers.Peer)
		 */
		public void requestComplete(PeerReadRequest arg0,
				PooledByteBuffer arg1, Peer arg2) {
			// TODO Auto-generated method stub
			
		}
		
	}
	
	public class SimulatedDownloadScrapeResult implements DownloadScrapeResult {

		
		public int totalNumKnownNonSeeds = -1;
		public int totalNumKnownSeeds = -1;

		public int getNonSeedCount() {
			return this.totalNumKnownNonSeeds;
		}
		
		public int getSeedCount() {
			return totalNumKnownSeeds;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadScrapeResult#getDownload()
		 */
		public Download getDownload() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadScrapeResult#getNextScrapeStartTime()
		 */
		public long getNextScrapeStartTime() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadScrapeResult#getResponseType()
		 */
		public int getResponseType() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadScrapeResult#getScrapeStartTime()
		 */
		public long getScrapeStartTime() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadScrapeResult#getStatus()
		 */
		public String getStatus() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadScrapeResult#getURL()
		 */
		public URL getURL() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadScrapeResult#setNextScrapeStartTime(long)
		 */
		public void setNextScrapeStartTime(long arg0) {
			// TODO Auto-generated method stub
			
		}
		
	}
	
	public class SimulatedTorrent implements Torrent {
		
		long pieceSize = -1;
		long size = -1;
		
		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getAdditionalProperty(java.lang.String)
		 */
		public Object getAdditionalProperty(String arg0) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getAnnounceURL()
		 */
		public URL getAnnounceURL() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getAnnounceURLList()
		 */
		public TorrentAnnounceURLList getAnnounceURLList() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getComment()
		 */
		public String getComment() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getCreatedBy()
		 */
		public String getCreatedBy() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getCreationDate()
		 */
		public long getCreationDate() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getEncoding()
		 */
		public String getEncoding() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getFiles()
		 */
		public TorrentFile[] getFiles() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getHash()
		 */
		public byte[] getHash() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getMagnetURI()
		 */
		public URL getMagnetURI() throws TorrentException {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getMapProperty(java.lang.String)
		 */
		public Map getMapProperty(String arg0) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getName()
		 */
		public String getName() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getPieceCount()
		 */
		public long getPieceCount() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getPieceSize()
		 */
		public long getPieceSize() {
			return pieceSize; 			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getPieces()
		 */
		public byte[][] getPieces() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getPluginStringProperty(java.lang.String)
		 */
		public String getPluginStringProperty(String arg0) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#getSize()
		 */
		public long getSize() {
			return size;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#isComplete()
		 */
		public boolean isComplete() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#isDecentralised()
		 */
		public boolean isDecentralised() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#isDecentralisedBackupEnabled()
		 */
		public boolean isDecentralisedBackupEnabled() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#isDecentralisedBackupRequested()
		 */
		public boolean isDecentralisedBackupRequested() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#isPrivate()
		 */
		public boolean isPrivate() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#isSimpleTorrent()
		 */
		public boolean isSimpleTorrent() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#removeAdditionalProperties()
		 */
		public Torrent removeAdditionalProperties() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#save()
		 */
		public void save() throws TorrentException {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setAnnounceURL(java.net.URL)
		 */
		public void setAnnounceURL(URL arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setComment(java.lang.String)
		 */
		public void setComment(String arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setComplete(java.io.File)
		 */
		public void setComplete(File arg0) throws TorrentException {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setDecentralisedBackupRequested(boolean)
		 */
		public void setDecentralisedBackupRequested(boolean arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setDefaultEncoding()
		 */
		public void setDefaultEncoding() throws TorrentEncodingException {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setEncoding(java.lang.String)
		 */
		public void setEncoding(String arg0) throws TorrentEncodingException {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setMapProperty(java.lang.String, java.util.Map)
		 */
		public void setMapProperty(String arg0, Map arg1) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setPluginStringProperty(java.lang.String, java.lang.String)
		 */
		public void setPluginStringProperty(String arg0, String arg1) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#setPrivate(boolean)
		 */
		public void setPrivate(boolean arg0) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#wasCreatedByUs()
		 */
		public boolean wasCreatedByUs() {
			// TODO Auto-generated method stub
			return false;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#writeToBEncodedData()
		 */
		public byte[] writeToBEncodedData() throws TorrentException {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#writeToFile(java.io.File)
		 */
		public void writeToFile(File arg0) throws TorrentException {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.torrent.Torrent#writeToMap()
		 */
		public Map writeToMap() throws TorrentException {
			// TODO Auto-generated method stub
			return null;
		}
		
	}
	
	public class SimulatedDownloadStats implements DownloadStats{
		
		public long downloadingTime = -1;
		public long seedingTime = -1;
		public long totalUploaded = -1;
		public long totalDownloaded = -1;
		public float availability = -1;
		public int completed = -1;
		public int discarded = -1;
		public long hashFails = -1;
		public int downloadRate = -1;
		public int uploadRate = -1;
		public int shareRatio = -1;		
		
		public long  getSecondsDownloading() {
			return downloadingTime;
		}
		
		public long getSecondsOnlySeeding() {
			return seedingTime;
		}
		
		public long getUploaded() {
			return totalUploaded;
		}
		
		public long getDownloaded() {
			return totalDownloaded;
		}

		public float getAvailability() {
			return availability;
		}

		public int getCompleted() {
			return completed;
		}

		public long getDiscarded() {
			return discarded;
		}

		public long getDownloadAverage() {
			return downloadRate;
		}

		public int getShareRatio() {
			return shareRatio;
		}

		public long getUploadAverage() {
			return uploadRate;
		}
		
		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getCheckingDoneInThousandNotation()
		 */
		public int getCheckingDoneInThousandNotation() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getDownloadCompleted(boolean)
		 */
		public int getDownloadCompleted(boolean arg0) {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getDownloadDirectory()
		 */
		public String getDownloadDirectory() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getETA()
		 */
		public String getETA() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getETASecs()
		 */
		public long getETASecs() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getElapsedTime()
		 */
		public String getElapsedTime() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getHashFails()
		 */
		public long getHashFails() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getHealth()
		 */
		public int getHealth() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getRemaining()
		 */
		public long getRemaining() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getSecondsSinceLastDownload()
		 */
		public long getSecondsSinceLastDownload() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getSecondsSinceLastUpload()
		 */
		public long getSecondsSinceLastUpload() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getStatus()
		 */
		public String getStatus() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getStatus(boolean)
		 */
		public String getStatus(boolean arg0) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getTargetFileOrDir()
		 */
		public String getTargetFileOrDir() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getTimeStarted()
		 */
		public long getTimeStarted() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getTimeStartedSeeding()
		 */
		public long getTimeStartedSeeding() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getTotalAverage()
		 */
		public long getTotalAverage() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getTrackerStatus()
		 */
		public String getTrackerStatus() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#resetUploadedDownloaded(long, long)
		 */
		public void resetUploadedDownloaded(long arg0, long arg1) {
			// TODO Auto-generated method stub
			
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getBytesUnavailable()
		 */
		public long getBytesUnavailable() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getDownloadAverage(boolean)
		 */
		public long getDownloadAverage(boolean arg0) {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getDownloaded(boolean)
		 */
		public long getDownloaded(boolean arg0) {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getUploadAverage(boolean)
		 */
		public long getUploadAverage(boolean arg0) {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadStats#getUploaded(boolean)
		 */
		public long getUploaded(boolean arg0) {
			// TODO Auto-generated method stub
			return 0;
		}

	}

	public class SimulatedDownloadAnnounceResult implements DownloadAnnounceResult {

		
		public int numKnownNonSeeds = -1;
		public int numKnownSeeds = -1;
		
		public int getNonSeedCount() {
			return numKnownNonSeeds;
		}
		
		public int getSeedCount() {
			return numKnownSeeds;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getDownload()
		 */
		public Download getDownload() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getError()
		 */
		public String getError() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getExtensions()
		 */
		public Map getExtensions() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getPeers()
		 */
		public DownloadAnnounceResultPeer[] getPeers() {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getReportedPeerCount()
		 */
		public int getReportedPeerCount() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getResponseType()
		 */
		public int getResponseType() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getTimeToWait()
		 */
		public long getTimeToWait() {
			// TODO Auto-generated method stub
			return 0;
		}

		/* (non-Javadoc)
		 * @see org.gudy.azureus2.plugins.download.DownloadAnnounceResult#getURL()
		 */
		public URL getURL() {
			// TODO Auto-generated method stub
			return null;
		}
		
	}
	
	public int getMaximumDownloadKBPerSecond() {
		return maximumDownloadKBPerSecond;
	}

	public String getName() {
		return name;
	}

	public SimulatedPeerManager getPeerManager() {
		return peerManager;
	}

	public SimulatedDownloadScrapeResult getLastScrapeResult() {
		return downloadScrapeResult;
	}

	public SimulatedDownloadAnnounceResult getLastAnnounceResult() {
		return downloadAnnounceResult;
	}
	
	public SimulatedDownloadStats getStats() {
		return stats;
	}
	
	public int getIndex() {
		return priority;
	}
	
	public int getPosition() {
		return position;
	}
	
	public int getState() {
		return state;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addActivationListener(org.gudy.azureus2.plugins.download.DownloadActivationListener)
	 */
	public void addActivationListener(DownloadActivationListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addAttributeListener(org.gudy.azureus2.plugins.download.DownloadAttributeListener, org.gudy.azureus2.plugins.torrent.TorrentAttribute, int)
	 */
	public void addAttributeListener(DownloadAttributeListener arg0,
			TorrentAttribute arg1, int arg2) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addCompletionListener(org.gudy.azureus2.plugins.download.DownloadCompletionListener)
	 */
	public void addCompletionListener(DownloadCompletionListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addDownloadWillBeRemovedListener(org.gudy.azureus2.plugins.download.DownloadWillBeRemovedListener)
	 */
	public void addDownloadWillBeRemovedListener(
			DownloadWillBeRemovedListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addListener(org.gudy.azureus2.plugins.download.DownloadListener)
	 */
	public void addListener(DownloadListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addPeerListener(org.gudy.azureus2.plugins.download.DownloadPeerListener)
	 */
	public void addPeerListener(DownloadPeerListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addPropertyListener(org.gudy.azureus2.plugins.download.DownloadPropertyListener)
	 */
	public void addPropertyListener(DownloadPropertyListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addTrackerListener(org.gudy.azureus2.plugins.download.DownloadTrackerListener)
	 */
	public void addTrackerListener(DownloadTrackerListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#addTrackerListener(org.gudy.azureus2.plugins.download.DownloadTrackerListener, boolean)
	 */
	public void addTrackerListener(DownloadTrackerListener arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removeActivationListener(org.gudy.azureus2.plugins.download.DownloadActivationListener)
	 */
	public void removeActivationListener(DownloadActivationListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removeAttributeListener(org.gudy.azureus2.plugins.download.DownloadAttributeListener, org.gudy.azureus2.plugins.torrent.TorrentAttribute, int)
	 */
	public void removeAttributeListener(DownloadAttributeListener arg0,
			TorrentAttribute arg1, int arg2) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removeCompletionListener(org.gudy.azureus2.plugins.download.DownloadCompletionListener)
	 */
	public void removeCompletionListener(DownloadCompletionListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removeDownloadWillBeRemovedListener(org.gudy.azureus2.plugins.download.DownloadWillBeRemovedListener)
	 */
	public void removeDownloadWillBeRemovedListener(
			DownloadWillBeRemovedListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removeListener(org.gudy.azureus2.plugins.download.DownloadListener)
	 */
	public void removeListener(DownloadListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removePeerListener(org.gudy.azureus2.plugins.download.DownloadPeerListener)
	 */
	public void removePeerListener(DownloadPeerListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removePropertyListener(org.gudy.azureus2.plugins.download.DownloadPropertyListener)
	 */
	public void removePropertyListener(DownloadPropertyListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadEventNotifier#removeTrackerListener(org.gudy.azureus2.plugins.download.DownloadTrackerListener)
	 */
	public void removeTrackerListener(DownloadTrackerListener arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#calculateDefaultDownloadLocation()
	 */
	public SaveLocationChange calculateDefaultDownloadLocation() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#calculateDefaultPaths(boolean)
	 */
	public File[] calculateDefaultPaths(boolean arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#canBeRemoved()
	 */
	public boolean canBeRemoved() throws DownloadRemovalVetoException {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#canMoveDataFiles()
	 */
	public boolean canMoveDataFiles() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#changeLocation(org.gudy.azureus2.plugins.download.savelocation.SaveLocationChange)
	 */
	public void changeLocation(SaveLocationChange arg0)
			throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getActivationState()
	 */
	public DownloadActivationEvent getActivationState() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute)
	 */
	public String getAttribute(TorrentAttribute arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getBooleanAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute)
	 */
	public boolean getBooleanAttribute(TorrentAttribute arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getCategoryName()
	 */
	public String getCategoryName() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getCreationTime()
	 */
	public long getCreationTime() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getDiskManager()
	 */
	public DiskManager getDiskManager() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getDiskManagerFileInfo()
	 */
	public DiskManagerFileInfo[] getDiskManagerFileInfo() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getDiskManagerFileInfo(int)
	 */
	public DiskManagerFileInfo getDiskManagerFileInfo(int arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getDownloadPeerId()
	 */
	public byte[] getDownloadPeerId() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getDownloadRateLimitBytesPerSecond()
	 */
	public int getDownloadRateLimitBytesPerSecond() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getErrorStateDetails()
	 */
	public String getErrorStateDetails() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getFlag(long)
	 */
	public boolean getFlag(long arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getFlags()
	 */
	public long getFlags() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getIntAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute)
	 */
	public int getIntAttribute(TorrentAttribute arg0) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getListAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute)
	 */
	public String[] getListAttribute(TorrentAttribute arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getLongAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute)
	 */
	public long getLongAttribute(TorrentAttribute arg0) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getMapAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute)
	 */
	public Map getMapAttribute(TorrentAttribute arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getPriority()
	 */
	public int getPriority() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getSavePath()
	 */
	public String getSavePath() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getSeedingRank()
	 */
	public int getSeedingRank() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getSubState()
	 */
	public int getSubState() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getTorrent()
	 */
	public SimulatedTorrent getTorrent() {
		return torrent;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getTorrentFileName()
	 */
	public String getTorrentFileName() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getUploadRateLimitBytesPerSecond()
	 */
	public int getUploadRateLimitBytesPerSecond() {
	
		return uploadRateLimitBytesPerSecond;
	
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getUserData(java.lang.Object)
	 */
	public Object getUserData(Object arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#hasAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute)
	 */
	public boolean hasAttribute(TorrentAttribute arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#initialize()
	 */
	public void initialize() throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isChecking()
	 */
	public boolean isChecking() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isComplete()
	 */
	public boolean isComplete() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isComplete(boolean)
	 */
	public boolean isComplete(boolean arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isForceStart()
	 */
	public boolean isForceStart() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isInDefaultSaveDir()
	 */
	public boolean isInDefaultSaveDir() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isMessagingEnabled()
	 */
	public boolean isMessagingEnabled() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isPaused()
	 */
	public boolean isPaused() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isPersistent()
	 */
	public boolean isPersistent() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isPriorityLocked()
	 */
	public boolean isPriorityLocked() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isRemoved()
	 */
	public boolean isRemoved() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isStartStopLocked()
	 */
	public boolean isStartStopLocked() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#moveDataFiles(java.io.File)
	 */
	public void moveDataFiles(File arg0) throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#moveDataFiles(java.io.File, java.lang.String)
	 */
	public void moveDataFiles(File arg0, String arg1) throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#moveDown()
	 */
	public void moveDown() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#moveTo(int)
	 */
	public void moveTo(int arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#moveTorrentFile(java.io.File)
	 */
	public void moveTorrentFile(File arg0) throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#moveUp()
	 */
	public void moveUp() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#pause()
	 */
	public void pause() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#recheckData()
	 */
	public void recheckData() throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#remove()
	 */
	public void remove() throws DownloadException, DownloadRemovalVetoException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#remove(boolean, boolean)
	 */
	public void remove(boolean arg0, boolean arg1) throws DownloadException,
			DownloadRemovalVetoException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#renameDownload(java.lang.String)
	 */
	public void renameDownload(String arg0) throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#requestTrackerAnnounce()
	 */
	public void requestTrackerAnnounce() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#requestTrackerAnnounce(boolean)
	 */
	public void requestTrackerAnnounce(boolean arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#requestTrackerScrape(boolean)
	 */
	public void requestTrackerScrape(boolean arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#restart()
	 */
	public void restart() throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#resume()
	 */
	public void resume() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setAnnounceResult(org.gudy.azureus2.plugins.download.DownloadAnnounceResult)
	 */
	public void setAnnounceResult(DownloadAnnounceResult arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute, java.lang.String)
	 */
	public void setAttribute(TorrentAttribute arg0, String arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setBooleanAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute, boolean)
	 */
	public void setBooleanAttribute(TorrentAttribute arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setCategory(java.lang.String)
	 */
	public void setCategory(String arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setDownloadRateLimitBytesPerSecond(int)
	 */
	public void setDownloadRateLimitBytesPerSecond(int arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setFlag(long, boolean)
	 */
	public void setFlag(long arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setForceStart(boolean)
	 */
	public void setForceStart(boolean arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setIntAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute, int)
	 */
	public void setIntAttribute(TorrentAttribute arg0, int arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setListAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute, java.lang.String[])
	 */
	public void setListAttribute(TorrentAttribute arg0, String[] arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setLongAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute, long)
	 */
	public void setLongAttribute(TorrentAttribute arg0, long arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setMapAttribute(org.gudy.azureus2.plugins.torrent.TorrentAttribute, java.util.Map)
	 */
	public void setMapAttribute(TorrentAttribute arg0, Map arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setMaximumDownloadKBPerSecond(int)
	 */
	public void setMaximumDownloadKBPerSecond(int arg0) {
		
		maximumDownloadKBPerSecond = arg0 * 1024;
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setMessagingEnabled(boolean)
	 */
	public void setMessagingEnabled(boolean arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setPosition(int)
	 */
	public void setPosition(int arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setPriority(int)
	 */
	public void setPriority(int arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setScrapeResult(org.gudy.azureus2.plugins.download.DownloadScrapeResult)
	 */
	public void setScrapeResult(DownloadScrapeResult arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setSeedingRank(int)
	 */
	public void setSeedingRank(int arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setUploadRateLimitBytesPerSecond(int)
	 */
	public void setUploadRateLimitBytesPerSecond(int arg0) {
		
		uploadRateLimitBytesPerSecond = arg0;
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#setUserData(java.lang.Object, java.lang.Object)
	 */
	public void setUserData(Object arg0, Object arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#start()
	 */
	public void start() throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#startDownload(boolean)
	 */
	public void startDownload(boolean arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#stop()
	 */
	public void stop() throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#stopAndQueue()
	 */
	public void stopAndQueue() throws DownloadException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#stopDownload()
	 */
	public void stopDownload() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getDiskManagerFileCount()
	 */
	public int getDiskManagerFileCount() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#addRateLimiter(org.gudy.azureus2.plugins.network.RateLimiter, boolean)
	 */
	public void addRateLimiter(RateLimiter arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#isMoving()
	 */
	public boolean isMoving() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#removeRateLimiter(org.gudy.azureus2.plugins.network.RateLimiter, boolean)
	 */
	public void removeRateLimiter(RateLimiter arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadStub#destubbify()
	 */
	public Download destubbify() throws DownloadException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadStub#getStubFiles()
	 */
	public DownloadStubFile[] getStubFiles() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadStub#getTorrentHash()
	 */
	public byte[] getTorrentHash() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadStub#getTorrentSize()
	 */
	public long getTorrentSize() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.DownloadStub#isStub()
	 */
	public boolean isStub() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#canStubbify()
	 */
	public boolean canStubbify() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#getAggregatedScrapeResult()
	 */
	public DownloadScrapeResult getAggregatedScrapeResult() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.plugins.download.Download#stubbify()
	 */
	public DownloadStub stubbify() throws DownloadException,
			DownloadRemovalVetoException {
		// TODO Auto-generated method stub
		return null;
	}

}
