/**
 * Dasu Project
 *
 * File:         DownloadStats.java
 * RCS:          $Id$
 * Description:  DownloadStats class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 10, 2007 at 1:39:15 PM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import java.lang.reflect.Field;
import java.util.Date;

import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadManager;

import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 *
 * localPeerStatDynamic, every 30 seconds, peerId, time, uploadRate (at that time), 
 * downloadRate (at that time), maxDonwloadRate, maxUploadRate, autoSeed
 */

public class DynamicLocalPeerStat extends FactTemplate implements Cloneable{
	
	private static final long serialVersionUID = 1L;
	
	public long currentTime = Util.currentGMTTime();
	public int uploadRate = -1;  	/*instantaneous upload rate*/
	public int downloadRate = -1;	/*instantaneous download rate*/
	public int maxUploadRate = -1;
	public int maxDownloadRate = -1;
	public boolean autoSpeed;
	public int rst;
	public int numConnections;
	public int passiveOpens;
	public int activeOpens;
	public int failedConns;
	public int numTorrents = 0;
	public int numLeechingTorrents = 0;
	public int numSeedingTorrents = 0;
	public int numConnectedSeeds = 0;
	public int numConnectedLeechers = 0;
	public long bytesSentLpsd;
	public long bytesReceivedLpsd;
	public long swarmPeerSpeed;
	public long bytesSentNetstat;
	public long bytesReceivedNetstat;
	public long UPnPNumberOfActiveConnections=-1;
	public long UPnPTotalBytesSent=-1;
	public long UPnPTotalBytesReceived=-1;
	public long UPnPTotalPacketsSent=-1;
	public long UPnPTotalPacketsReceived=-1;
	public long UPnPUptime=-1;
	public float dasuCpuUtilization = -1; /*instantaneous cpu utilization*/
	public float jvmCpuUtilization = -1; /*instantaneous cpu utilization*/
	
	public int maxUploadRateLimitAllActiveTorrents = 0;
	public int maxDownloadRateLimitAllActiveTorrents = 0;

	public DynamicLocalPeerStat() {
	}
	
	public DynamicLocalPeerStat(long currentTime) {
		
		super();
		
		if ( DasuManager.USE_SIMULATION_V2) {
			useSimulation();
		}
		else {
			DownloadManager dm = Main.getPluginInterface().getDownloadManager();
					
			downloadRate = dm.getStats().getDataReceiveRate();
			uploadRate = dm.getStats().getDataSendRate();
			maxDownloadRate = COConfigurationManager.getIntParameter("Max Download Speed KBs")*1000;
			maxUploadRate = COConfigurationManager.getIntParameter("Max Upload Speed KBs")*1000;
			autoSpeed = checkAutoSpeed();
			numTorrents = dm.getDownloads().length;
			bytesSentLpsd = dm.getStats().getDataBytesSent();
			bytesReceivedLpsd = dm.getStats().getDataBytesReceived();
			swarmPeerSpeed = Main.getGlobalManager().getStats().getTotalSwarmsPeerRate(true, false);		
	
			if ( DasuManager.getResourceProfiler() != null) {
				dasuCpuUtilization = DasuManager.getResourceProfiler().getDasuCpuUsage();
				jvmCpuUtilization = DasuManager.getResourceProfiler().getJvmCpuUsage();				
			}
			
			numConnectedSeeds = 0;
			numConnectedLeechers = 0;
			for (Download d : dm.getDownloads()) {
				
				maxUploadRateLimitAllActiveTorrents += d.getUploadRateLimitBytesPerSecond();
	
				if (d.getState() == Download.ST_SEEDING) {
					numSeedingTorrents++;
				}
				if (d.getState() == Download.ST_DOWNLOADING) {
					numLeechingTorrents++;
					maxDownloadRateLimitAllActiveTorrents += d.getMaximumDownloadKBPerSecond()*8;					
				}
				
				if (d.getPeerManager() != null) {
					numConnectedSeeds += d.getPeerManager().getStats().getConnectedSeeds();
					numConnectedLeechers += d.getPeerManager().getStats().getConnectedLeechers();
				}
			}	
		}

	}
	
	public void useSimulation() {
			
		DynamicLocalPeerStat dlpsd = (DynamicLocalPeerStat) DasuManager.interferenceSimulator.iterGetStat(DynamicLocalPeerStat.class);
		if ( dlpsd != null) {
			downloadRate = dlpsd.downloadRate;
			uploadRate = dlpsd.uploadRate;
			maxDownloadRate = dlpsd.maxDownloadRate;
			maxUploadRate = dlpsd.maxUploadRate;
			autoSpeed = dlpsd.autoSpeed;
			numTorrents = dlpsd.numTorrents;
			bytesSentLpsd = dlpsd.bytesSentLpsd;
			bytesReceivedLpsd = dlpsd.bytesReceivedLpsd;
			swarmPeerSpeed = dlpsd.swarmPeerSpeed;		
			numConnectedSeeds = dlpsd.numConnectedSeeds;
			numConnectedLeechers = dlpsd.numConnectedLeechers;    				
			maxUploadRateLimitAllActiveTorrents = dlpsd.maxDownloadRateLimitAllActiveTorrents;
			numSeedingTorrents = dlpsd.numSeedingTorrents;
			numLeechingTorrents = dlpsd.numLeechingTorrents;
			maxDownloadRateLimitAllActiveTorrents = dlpsd.maxDownloadRateLimitAllActiveTorrents;
			
			System.out.println("DynamicLocalPeerStat: USE SIMULATION!***************************** " + bytesSentLpsd + " (" + new Date(currentTime) + ") " + currentTime);
		}
		else {		
			System.out.println("DynamicLocalPeerStat: USE SIMULATION!***************************** " + null);
		}
				
	}


	private boolean checkAutoSpeed() {
		String config = "";
		if ( COConfigurationManager.getBooleanParameter(TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY)){
			
			config = ( TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY );
		} else {
		
	    	if ( Main.getPluginInterface().getDownloadManager().isSeedingOnly()){
	        	
	    		config = ( TransferSpeedValidator.AUTO_UPLOAD_SEEDING_ENABLED_CONFIGKEY );
	        	
	      	}else{
	      		
	      		config = ( TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY );
	      	}
		}
		
		return COConfigurationManager.getBooleanParameter(config);
	}
	
    public Object clone() {

   	 try {
             return super.clone();
         }
   	 catch( CloneNotSupportedException e ) {
             return null;
         }
     } 
    
    @Override
    public String toString() {
    	
    	String ret = "DynamicLocalPeerStat: <";
    	try {
        	
    		for ( Field f : this.getClass().getFields()) {
    		
    			//System.out.println(f.getName() + " " + f.get(this));
    			if ( f.get(this) != null)
    				ret = ret.concat(f.getName() + ": " + f.get(this).toString() + " ");
    		}
    		
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    	
    	ret = ret.concat(">");
    	return ret;
    }

}