/**
 * Dasu Project
 *
 * File:         Statistics.java
 * RCS:          $Id$
 * Description:  Statistics class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jun 14, 2006 at 7:51:32 AM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import edu.northwestern.cs.aqualab.cattle.Report;
import edu.northwestern.cs.aqualab.cattle.ReportManager;
import edu.northwestern.cs.aqualab.cattle.jsonsimple.JSONObject;
import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.timer.DasuTimerManager;

/**
 * @author Mario Sanchez
 * 
 *         The Statistics class manages statistics collected by this plugin.
 */
public class Statistics {
	
	private static Statistics self;
	public final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");	
	private boolean DEBUG = false;
	
	private Statistics() {
				
	}

	public static synchronized Statistics getInstance() {
		
		if (self == null) {
			self = new Statistics();
		}

		return self;
	}
	
	public void addReportableFact( FactTemplate fpr ) {
		
		if ( DasuConfiguration.getInstance().isSendToDatabase() ) {			
			reportWithWrangler(fpr);			
		}
		else {
			if (DEBUG)
				LOGGER.info("REPORTER: drop report, data reporting has been disabled by user");
		}
	}
	
	public void reportWithWrangler( FactTemplate fpr ) {
		
		String type = fpr.getClass().getSimpleName();
		if ( fpr instanceof ReportableFact )
			type = ((ReportableFact) fpr).getFactType();
		
		Report report = new Report(type);
		JSONObject json = fpr.toJSON();
		if (!DasuConfiguration.getInstance().isEnableExecutionPathTimestamps()) {
			if (json.containsKey("executionPathTimestamps"))
				json.remove("executionPathTimestamps");
		}		
		report.setJSONObject(json);
		
		try { 
			ReportManager.getInstance();
		}
		catch (Exception e) {
			return;
		}
		ReportManager.getInstance().send(report);

	}
		
	public void stop() {
		DasuTimerManager.getInstance().destroyTimer("DasuStatistics");					
	}

		
}

