/**
 * Dasu Project
 *
 * File:         DasuTimerManager.java
 * RCS:          $Id: DasuTimerManager.java,v 1.0 Feb 18, 2013 10:13:56 AM mas939 Exp $
 * Description:  DasuTimerManager class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 18, 2013 at 10:13:56 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.timer
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2013, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.timer;

import java.util.*;
import java.util.Map.Entry;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The DasuTimerManager class ...
 */
public class DasuTimerManager {
	
	public final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	private Hashtable<String, AzureusTimer> timers;
	private static DasuTimerManager self;
	
	public DasuTimerManager(){
		timers = new  Hashtable<String, AzureusTimer>();
	}
	
	public static DasuTimerManager getInstance(){
		if (self==null){
			self = new DasuTimerManager();
		}
		return self;
	}

	//override and destroy any previously existing timers with the same name
	private ITimer createTimer(String name){
		AzureusTimer st = new AzureusTimer(Main.getPluginInterface().getUtilities().createTimer(name), name);
		
		synchronized(timers) {
			ITimer old = timers.get(name);
			if (old!=null){
				((ITimer)old).destroy();
			}
			timers.put(name, st);
		}
		return st;
		
	}
	
	public boolean createEvent(String timerName, TimerEntry when, ITimerEventPerformer performer) {
		
		ITimer timer = createTimer(timerName);
		if ( !when.isPeriodic()) {
			
			long curr = Util.currentTimeMillis();
			//if the specified time is more than 5 seconds in the past
			if ( when.getEventTime() <  curr-5*1000 ) {
				LOGGER.warning("TIMER: time is in the past, ignore");
				return false;
			}
			
			LOGGER.info("TIMER: registering non-periodic event " + timerName);
			timer.addEvent(when.getEventTime(), performer);
		}
		else {
			LOGGER.info("TIMER: registering periodic event " + timerName + " in " + when.getEventTime() + " millis");
			timer.addPeriodicEvent(when.getEventTime(), performer);
		}
		
		return true;
		
	}

	public void stopManager() {
		
		LOGGER.info("STOP: stopping all timers");
		synchronized(timers) {
			Iterator<Entry<String, AzureusTimer>> iter = timers.entrySet().iterator();
			while ( iter.hasNext() ) {
				Entry<String, AzureusTimer> entry = iter.next();
				entry.getValue().destroy();
				iter.remove();
			}
		}
	}
	
	public void destroyTimer(String name) {
		synchronized(timers) {
			if ( timers.containsKey(name) ) {
				ITimer timer = timers.remove(name);
				timer.destroy();
			}
			
		}
	}
	

	
}
