/**
 * Dasu Project
 *
 * File:         TimerEntry.java
 * RCS:          $Id: TimerEntry.java,v 1.0 Feb 18, 2013 9:31:05 AM mas939 Exp $
 * Description:  TimerEntry class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 18, 2013 at 9:31:05 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.timer
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2013, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.timer;

import java.util.Calendar;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The TimerEntry class ...
 */
public class TimerEntry {

	private Calendar eventCal = Calendar.getInstance();
	private long eventTime = -1;
	private boolean periodic = false;

	public TimerEntry(long milliseconds, boolean periodic) {
		
		eventTime = milliseconds;
		this.periodic = periodic;
		
	}	

	public TimerEntry(int year, int month, int dayOfMonth, int hourOfDay, int minute, int second, int milliseconds) {
		 
		eventCal.set(Calendar.HOUR_OF_DAY, hourOfDay);
		eventCal.set(Calendar.MINUTE, minute);
		eventCal.set(Calendar.SECOND, second);
		eventCal.set(Calendar.DAY_OF_MONTH, dayOfMonth);
		eventCal.set(Calendar.MONTH, month);
		eventCal.set(Calendar.YEAR, year);
		eventCal.set(Calendar.MILLISECOND, milliseconds);
		periodic = false;
		
	}
	
	public TimerEntry(int year, int month, int dayOfMonth, int hourOfDay, int minute, int second, int milliseconds,
			boolean isGMT) {

		this(year,month,dayOfMonth,hourOfDay,minute,second,milliseconds);
		// adjust GMT to localtime equivalent
		if ( isGMT ) {
			long offset = Util.currentDasuTime() - Util.currentGMTTime() + Util.getDasuSntpTimeOffset();
			eventCal.add(Calendar.MILLISECOND, (int) offset);
		}
		
	}
	
	public TimerEntry(Calendar time) {
		eventCal = time;
	}
	
	public long getEventTime() {
		
		if (!isPeriodic())
			return eventCal.getTimeInMillis();
		
		return eventTime;
	}
	
	public boolean isPeriodic() {
		return periodic;
	}

}
