/**
 * Dasu Project
 *
 * File:         DasuView.java
 * RCS:          $Id$
 * Description:  DasuView class (see below)
 * Author:       Zachary Bischof
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 26, 2010 at 12:21:13 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.ui
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.ui;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Logger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.AbstractHandler;
import org.eclipse.jetty.server.handler.ResourceHandler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.browser.CloseWindowListener;
import org.eclipse.swt.browser.OpenWindowListener;
import org.eclipse.swt.browser.TitleEvent;
import org.eclipse.swt.browser.TitleListener;
import org.eclipse.swt.browser.VisibilityWindowListener;
import org.eclipse.swt.browser.WindowEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.gudy.azureus2.ui.swt.Utils;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;

import com.sun.net.httpserver.HttpServer;

import edu.northwestern.cs.aqualab.cattle.jsonsimple.JSONObject;
import edu.northwestern.cs.aqualab.cattle.jsonsimple.parser.JSONParser;
import edu.northwestern.cs.aqualab.cattle.jsonsimple.parser.ParseException;
import edu.northwestern.dasu.DasuData;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactFireAction;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.stats.Statistics;


/*

// How to update the ISP
String prefix = NetworkAdminASNLookupImpl.getInstance().getASNInfo().getBGPPrefix();
DasuManager.getPersistentDataManager().setIspInfo(new Pair(ispName, Util.currentGMTTime()));
Statistics.getInstance().commitIspInfo(prefix,ispName);

// Report undetected interference
Statistics.getInstance().commitUserComments(interferenceReportTorrentIds, interferenceReportComments);

 */

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The DasuView class ...
 */

public class DasuView implements UISWTViewEventListener {

	public final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");

	// Constants
	public static final String VIEWID = "Dasu";
	private static final String jsname = "dasu_";
	private static final int basePort = 30000;
	// number of connections to queue. since httpserver is
	//single threaded (I think), this should be reasonably high.

	// Objects
	public DasuData dd;
	private static DasuView self;
	private Display display;
	private Browser browserUi = null;
	private static ArrayList<Browser> browsers = new ArrayList<Browser>();

	// State
	protected boolean starting = false;
	protected boolean failed = false;

	private boolean isCreated = false;
	private boolean isLoaded = false;
	private int serverPort = -1;

	private AbstractHandler handler;
	HttpServer s = null;
	private HashMap<String, ByteArrayOutputStream> uiFiles = null;

	/**
	 * @return Returns value of isLoaded.
	 */
	public boolean isLoaded() {
		return isLoaded;
	}

	public DasuView() {
		self = this;

		// Register the UI with DasuData
		dd = DasuData.getInstance();
		dd.setUi(this);
		
		if (!java.awt.Desktop.isDesktopSupported()) {
			LOGGER.info("PROFILER: ThreadCpuTime is not supported!!");
			ReportableFact rf = new ReportableFact("ErrorEvent");
			rf.addContentsObject("event", "awt Desktop API not supported!");
			Statistics.getInstance().addReportableFact(rf);
		}
	}


	public abstract class DasuPageHandler extends AbstractHandler{
		
		public void handle(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response)
						throws IOException, ServletException {
								
			if ( !target.contains("/ajax/") ) {
				handleFileRequest(target, baseRequest, request, response);
			}
			else
				handleAjaxRequest(target, baseRequest, request, response);
						
		}
		
		public abstract void handleFileRequest(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response)
						throws IOException, ServletException;

		public void handleAjaxRequest(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {
			
			if ( request.getParameter("func").equals("dasu_loaded")) {
				handleDasuLoadedFunction(target, baseRequest, request, response);
			}
			else if ( request.getParameter("func").equals("dasu_update")) {
				handleUpdateFunction(target, baseRequest, request, response);
			}
			else if ( request.getParameter("func").equals("dasu_fireAction")) {
				handleFireActionFunction(target, baseRequest, request, response);
			}
			else if ( request.getParameter("func").equals("dasu_error")) {
				handleErrorFunction(target, baseRequest, request, response);
			}
			else if ( request.getParameter("func").equals("dasu_updateUi")) {
				handleDasuUpdateUiFunction(target, baseRequest, request, response);
			}
			else if ( request.getParameter("func").equals("dasu_openWebLink")) {
				handleOpenWebLink(target, baseRequest, request, response);
			}

			
		}
		
		public void handleDasuLoadedFunction(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {
						
			
				LOGGER.info("UI: " + request.getParameterMap());
				LOGGER.info("UI: ui loaded!");
				JSONObject json = new JSONObject();
				
				isLoaded = true;
				Locale def = Locale.getDefault();
				String defStr = "";
				if(def != null) {
					defStr = def.toString();
				}
				json.put("ui_locale", defStr);
				//enable debugging...
				if(Main.DEBUG) {
					LOGGER.info("enabling debug mode!");
					json.put("enable_debug", true);
				}				
				
				sendResponse(response, baseRequest, json);
			
		}
		
		public void handleDasuUpdateUiFunction(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {
						
			
				LOGGER.info("UI: " + request.getParameterMap());

				JSONObject json = new JSONObject();
				// flush values stored locally
				Map<String, String> values = DasuData.getInstance().getAllValues();
				for (String key : values.keySet() ) {
					try {
						json.put(key, new JSONParser().parse(values.get(key)));
					} catch (ParseException e) {
						e.printStackTrace();
					}
				}
			
				sendResponse(response, baseRequest, json);
		}

		/**
		 * Accept updates to variables, pushed from the UI.
		 */
		public void handleUpdateFunction(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {
			
				LOGGER.info("UI: " + request.getParameterMap());
				boolean success = true;
				
				try {
					String key = request.getParameter("key");					
					Object newVal = new JSONParser().parse(request.getParameter("value"));				
					DasuData.getInstance().updateFromUi(key, newVal);					
				} catch (Exception e) {
					success = false;
					LOGGER.info("UI: Error: " + e.getMessage());
				}
				
				JSONObject json = new JSONObject();
				json.put("success", success);			
				sendResponse(response, baseRequest, json);
				
		}
		
		public void handleFireActionFunction(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {

				LOGGER.info("UI: " + request.getParameterMap());
				
				boolean success = true;
				
				try {					
					String action = request.getParameter("action");
					String args = request.getParameter("arguments");
					Object arguments = null;
					if ( args != null && args.length() > 0 )
						arguments = new JSONParser().parse(args);
					LOGGER.info("UI: fire action type " + action + " action " + arguments);
					AMCoordinatorV2.assertToEngine(SourceType.RE, new FactFireAction(action, (String) arguments));
					
				} catch (Exception e) {
					LOGGER.warning("UI: Error: " + request.getParameterMap());
					success = false;					
				}
				
				JSONObject json = new JSONObject();
				json.put("success", success);			
				sendResponse(response, baseRequest, json);
				
		}

		public void handleErrorFunction(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {

				LOGGER.info("UI: " + request.getParameterMap());
				
				boolean success = true;
				
				try {			
					String msg = request.getParameter("message");
					String url = request.getParameter("url");
					String line = request.getParameter("line");
										
					LOGGER.warning("UI: Javascript error in [" + url + "] on line [" + line + "]: " + msg);
					
				} catch (Exception e) {
					LOGGER.warning("UI: Error: " + e.getMessage());
					success = false;
				}
				
				JSONObject json = new JSONObject();
				json.put("success", success);			
				sendResponse(response, baseRequest, json);

		}
		
		public void handleOpenWebLink(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {

				LOGGER.info("UI: " + request.getParameterMap());
				
				boolean success = true;
				
				try {			
					
					String url = request.getParameter("url");
					if (java.awt.Desktop.isDesktopSupported()) {						
						java.awt.Desktop desktop = java.awt.Desktop.getDesktop();
						URI uri = new URI(url);
						desktop.browse(uri);
					}
					
				} catch (Exception e) {
					LOGGER.warning("UI: Error: " + e.getMessage());
					success = false;
				}
				
				JSONObject json = new JSONObject();
				json.put("success", success);			
				sendResponse(response, baseRequest, json);

		}
				
		private void sendResponse(HttpServletResponse response, 
				Request baseRequest,
				JSONObject json) throws IOException, ServletException {
			
			response.setContentType("application/json;charset=utf-8");
			response.setHeader("Cache-Control", "no-cache");
			response.setStatus(HttpServletResponse.SC_OK);

			LOGGER.info("UI: response: " + json.toJSONString());
			response.setContentLength(json.toJSONString().length());
			response.getWriter().write(json.toJSONString());
			response.flushBuffer();
			baseRequest.setHandled(true);

		}

		
	}
	
	public class ZipPageHandlerJetty extends DasuPageHandler {
				
		public void handleFileRequest(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {
			
			String str = target.substring(1);
			if(!uiFiles.containsKey(str)){
				response.setStatus(HttpServletResponse.SC_NOT_FOUND);
				baseRequest.setHandled(true);
				return;
			}
			response.setStatus(HttpServletResponse.SC_OK);			
			baseRequest.setHandled(true);
			response.getOutputStream().write(uiFiles.get(str).toByteArray());

		}
	}
	
	public class DefaultResourceHalnderJetty extends DasuPageHandler {
		
		ResourceHandler handler = new ResourceHandler();
		
		public DefaultResourceHalnderJetty(String file) {
			LOGGER.severe("UI: " + file);
			handler.setDirectoriesListed(true);
			handler.setResourceBase(file);
		}

		public void handleFileRequest(String target,
				Request baseRequest,
				HttpServletRequest request,
				HttpServletResponse response) throws IOException, ServletException {
			
			handler.handle(target, baseRequest, request, response);
		}
		
	}

	public int startHttpServer(int port, AbstractHandler handler) {
		Server server = null;
		int failures = 0;
		while(true) {
			try{				
				server = new Server(port);
				server.setHandler(handler);
				server.start();
				return port;
			} catch (Exception e) {
				try {
					server.stop();
				} catch (Exception e1) {}
				port += 1;
				if(failures > 100){
					return -1;
				}
			}
		}
	}


	public static DasuView getInstance() {
		return self;
	}

	public String getFullTitle() {
		return "Dasu";
	}

	public String getData() {
		return "Dasu";
	}

	public boolean eventOccurred(UISWTViewEvent event) {

		switch (event.getType()) {
		case UISWTViewEvent.TYPE_CREATE:
			if (isCreated){
				return false;
			}
			isCreated = true;
			break;

		case UISWTViewEvent.TYPE_DESTROY:
			delete();
			break;

		case UISWTViewEvent.TYPE_INITIALIZE:
			initialize((Composite)event.getData());
			break;

		//case UISWTViewEvent.TYPE_LANGUAGEUPDATE:
		//	if(isLoaded) asyncExecJs("ui_setLocale('" + Locale.getDefault().toString() + "')");
		//	break;
		}
			
		return true;
	}

	public void delete() {
		browserUi = null;
		display = null;
		isLoaded = false;
		isCreated = false;
	}


	static void initialize(final Display display, Browser browser) {

		browser.addOpenWindowListener(new OpenWindowListener(){

			public void open(WindowEvent event) {

				// Certain platforms can provide a default full browser.
				// simply return in that case if the application prefers
				// the default full browser to the embedded one set below.
				if (!event.required) return;

				// Embed the new window
				Shell shell = new Shell(display);
				shell.setText("New Window2"); 
				shell.setSize(400, 300); 

				shell.setLayout(new FillLayout());
				Browser browser2 = new Browser(shell, SWT.NONE);
				browser2.setJavascriptEnabled(true);
				initialize(display, browser2);

				DasuView.getInstance().new InsideDasuFunction(browser2, jsname + "insider");

				event.browser = browser2;
				synchronized(browsers) {
					browsers.add(browser2);
				}

				//System.out.println("browser: " + browsers.size() + "  " + browsers);

			}});	

		browser.addVisibilityWindowListener(new VisibilityWindowListener() {
			public void hide(WindowEvent event) {
				Browser browser = (Browser)event.widget;
				Shell shell = browser.getShell();
				shell.setVisible(false);
			}
			public void show(WindowEvent event) {

				Browser browser = (Browser)event.widget;
				Shell shell = browser.getShell();
				if (event.location != null) shell.setLocation(event.location);                                                
				if (event.size != null) {
					Point size = event.size;
					shell.setSize(shell.computeSize(size.x, size.y));
				}

				if (event.addressBar || event.menuBar || event.statusBar || event.toolBar) {
					// Create widgets for the address bar, menu bar, status bar and/or tool bar
					// leave enough space in the Shell to accommodate a Browser of the size
					// given by event.size                        	

				}
				shell.open();
			}
		});

							browser.addCloseWindowListener(new CloseWindowListener() {
								public void close(WindowEvent event) {
									Browser browser = (Browser)event.widget;
									Shell shell = browser.getShell();
									shell.close();
								}
							});

							browser.addTitleListener(new TitleListener() {

								public void changed(TitleEvent event) {

									Browser browser = (Browser)event.widget;
									Shell shell = browser.getShell();
									shell.setText(event.title);

								}

							});


	}

	public void addOpenWindowListener(Browser browser, final Display display) {

		browser.addOpenWindowListener(new OpenWindowListener(){

			public void open(WindowEvent event) {

				// Certain platforms can provide a default full browser.
				// simply return in that case if the application prefers
				// the default full browser to the embedded one set below.
				if (!event.required) return;

				// Embed the new window
				Shell shell = new Shell(display);
				shell.setText("New Window"); 
				shell.setSize(400, 300); 

				shell.setLayout(new FillLayout());
				Browser browser = new Browser(shell, SWT.NONE);
				browser.setJavascriptEnabled(true);
				initialize(display, browser);

				DasuView.getInstance().new InsideDasuFunction(browser, jsname + "insider");
				//browser.setSize(200, 200);
				event.browser = browser;
				synchronized(browsers) {
					browsers.add(browser);
				}

			}});	
		browser.addCloseWindowListener(new CloseWindowListener(){

			public void close(WindowEvent event) {
				Browser browser = (Browser)event.widget;
				Shell shell = browser.getShell();
				shell.close();
			}

		});

	}

	public void initialize(Composite composite) {
		
		display = composite.getDisplay();
//		Composite comp = new Composite(composite, SWT.EMBEDDED | SWT.NO_BACKGROUND );		
//		comp.setLayout(new FillLayout());
//	    Frame frame = org.eclipse.swt.awt.SWT_AWT.new_Frame(comp);
//	    frame.setSize(composite.getSize().x, composite.getSize().y);
//	    
//	    BrowserManager browserManager = new JBrowserBuilder().buildBrowserManager();
//        JComponentFactory<Canvas> canvasFactory = browserManager.getComponentFactory(JBrowserCanvas.class);
//        final JBrowserComponent<?> webBrowser = canvasFactory.createBrowser();        
//        System.out.println(canvasFactory.getBrowserManager().getStatus());
//        System.out.println(canvasFactory.getBrowserManager().getError());
//	    frame.add(webBrowser.getComponent());	    	    
	    
		browserUi = Utils.createSafeBrowser(composite, SWT.NONE);
		browserUi.setJavascriptEnabled(true);
		
		if ( DasuManager.USE_SIMULATION_V2)
			addOpenWindowListener(browserUi, display);

		// Register browser functions
		// http://dev.eclipse.org/viewcvs/viewvc.cgi/org.eclipse.swt.snippets/src/org/eclipse/swt/snippets/Snippet307.java?view=co
		DasuView.getInstance().new InsideDasuFunction(browserUi, jsname + "insider");
		
		if( !(starting || failed) && serverPort == -1) {
			starting = true;
			// Only do this once!
			if(Main.SKIP_UPDATE_UI_RULES) {
				
				LOGGER.severe("OK if debugging, not for release! Not serving UI from ZIP, or doing anything to keep the UI updated!");
				handler = new DefaultResourceHalnderJetty(Main.getDir() + java.io.File.separator + "ui");
				serverPort = startHttpServer(basePort, handler);
				starting = false;				
				
				//browserUi.setUrl("http://google.com");
				//browserUi.setUrl("http://localhost:" + serverPort + "/index.html");				
				browserUi.setUrl("http://localhost:" + serverPort + "/index_about.html");
				
			} else {
				Thread t = new Thread() {
					public void run() {
						LOGGER.info("UI: Waiting for updater to finish before starting the UI");
						try {
							// Wait for the updater to complete
							while(DasuManager.updater == null || !DasuManager.updater.isReady()) {								
								Thread.sleep(500);

								if ( Main.isShuttingDown() )
									interrupt();
							}
						} catch (InterruptedException e) {
							e.printStackTrace();
							LOGGER.severe("UI: DasuView interrupted while waiting for DasuManager.updater to become ready...");
							failed = true;
							starting = false;
							return;
						}		
						LOGGER.info("UI: Updater finished, UI can be started");
										// Load and serve from ZIP file
						uiFiles = DasuManager.updater.getUiData();
						handler = new ZipPageHandlerJetty();
						serverPort = startHttpServer(basePort, handler);
						starting = false;

						display.syncExec(new Runnable() {							
							public void run() {
								//browserUi.setUrl("http://google.com");
								browserUi.setUrl("http://localhost:" + serverPort + "/index.html");
								browserUi.setUrl("http://localhost:" + serverPort + "/index_about.html");
							}
						});
					}
				};
				t.setName("Dasu::ViewWaitThread");
				t.start();
			}
		} else {
			
			display.syncExec(new Runnable() {
				public void run() {
					//browserUi.setUrl("http://google.com");
					//browserUi.setUrl("http://localhost:" + serverPort + "/index.html");
					browserUi.setUrl("http://localhost:" + serverPort + "/index_about.html");					
				}
			});
		}
		
				
	}

	class InsideDasuFunction extends BrowserFunction {
		InsideDasuFunction (Browser browser, String name) {
			super (browser, name);
		}
		
		public Object function (Object[] arguments) {
			return true;
		}
	}

	public void stopDasuView() {

		if ( s != null)
			s.stop(0);
	}
}
