/**
 * Dasu Project
 *
 * File:         DasuClassLoader.java
 * RCS:          $Id: DasuClassLoader.java,v 1.0 Aug 9, 2012 10:17:23 AM mas939 Exp $
 * Description:  DasuClassLoader class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Aug 9, 2012 at 10:17:23 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.sandbox
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.util;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The DasuClassLoader class ...
 */

public class DasuClassLoader extends ClassLoader{

	public final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	private String jarName;
	private JarFile jar;
	Hashtable<String, Class> loadedClasses = new Hashtable();
	
    public DasuClassLoader(ClassLoader parent, String jarFile) {
        super(parent);
        updateJarFile(jarFile);
    }
    
    public void updateJarFile(String file) {
    	jarName = file;
    	
    	LOGGER.info("CLASSLOADER: " + jarName);
        try {
			jar = new JarFile(jarName);

        } catch (Exception e) {
			LOGGER.severe("CLASSLOADER: could not open " + jarName);
		}
        
    }
        
    public Class loadClass(String className) throws ClassNotFoundException {
    	
    	try {
    		
    		// Step 1 -- Check for a previously loaded class    		
        	if (loadedClasses.containsKey(className)) {
        		LOGGER.info("CLASSLOADER: already loaded! " + className);
        		return loadedClasses.get(className);
        	}
        	
        	Class cl = findLoadedClass(className);
        	if ( cl != null )
        		return cl;
        	
            if ( className.contains("dynamic") ) {
            	
            	try {
            		
            		String clazzName = className.replace(".", "/") + ".class";
	                JarEntry entry = jar.getJarEntry(clazzName);	                	                
	                InputStream is = jar.getInputStream(entry);  
	                ByteArrayOutputStream byteStream = new ByteArrayOutputStream();  
	                int nextValue = is.read();  
	                while (-1 != nextValue) {  
	                    byteStream.write(nextValue);  
	                    nextValue = is.read();  
	                }
	      
	                byte[] classByte = byteStream.toByteArray();  
	                Class result = defineClass(className, classByte, 0, classByte.length, null);  
	                loadedClasses.put(className, result);
	                return result;
	                
            	}
            	catch (Exception e) {             		
            		//pass            	
            	}
                
            }

		} catch (Exception e2) {
			e2.printStackTrace();
		}
    	    	        
    	return super.loadClass(className);        	
        
    }
    
    public byte[] getClassByte(String className) throws ClassNotFoundException {
    	
    	try {
    		
    		String clazzName = className.replace(".", "/") + ".class";
            JarEntry entry = jar.getJarEntry(clazzName);	                	                
            InputStream is = jar.getInputStream(entry);  
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();  
            int nextValue = is.read();  
            while (-1 != nextValue) {  
                byteStream.write(nextValue);  
                nextValue = is.read();  
            }
  
            byte[] classByte = byteStream.toByteArray();
            return classByte;
            
    	}
    	catch (Exception e) { 
    		//pass
    		throw new ClassNotFoundException();
    	}        
        
    }
    
    public void loadExternalClasses() {
    	
		FindClass find = new FindClass();
		//jarName = "/Users/mas939/Library/Application Support/Vuze/plugins/dasu/DasuUiRulesModules_4.2.jar";
		find.addLocations(jarName);
		find.addFile(".class");
		find.setRecurse(true);
		find.findFiles();


		try {
			
			Class cls = Class.forName("java.lang.ClassLoader");
		    Method m = cls.getDeclaredMethod("defineClass", new Class[] { String.class, byte[].class, int.class, int.class });
			m.setAccessible(true);
	
				for (String fqn : find.fqn_matches ) {
					
					try {
						fqn = fqn.replace(".class", "");
					    
					    LOGGER.info("DYNAMIC: loading class from external jar: " + fqn);
						byte[] classBytes = getClassByte(fqn);
						Object o = m.invoke(getParent(), new Object[]{fqn,classBytes,0,classBytes.length});						
					}
					catch (Exception e) {
						//e.printStackTrace();
						LOGGER.info("ERROR: DYNAMIC: " + e.getMessage());
					}
	
				}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
    	
    }
    
    public InputStream getResourceAsStream(String name) {
    	
    	try {

			String clazzName = name.replace(".", "/");
			if ( clazzName.endsWith("/exe") )
				clazzName = name.replace("/exe", ".exe");
	        JarEntry entry = jar.getJarEntry(clazzName);	                	                
	        InputStream is = jar.getInputStream(entry);
	        return is;
    	}
    	catch (Exception e) {    		
    		return super.getResourceAsStream(name);
    	}
    	    	
    }
    
    public static void main(String[] args) {
    	
    	DasuClassLoader d = new DasuClassLoader(ClassLoader.getSystemClassLoader(),"/Users/mas939/Library/Application Support/Vuze/plugins/dasu/DasuUiRulesModules_4.2.jar");
    	d.loadExternalClasses();
    	
    	/*
    	System.out.println("\n\n");
    	
    	try {
    		
    		JarFile jf = new JarFile("/tmp/DasuUiRulesModules.jar");
    		
	   		 KeyStore ks = null;
	         ks = KeyStore.getInstance("JKS");
	         InputStream is = new FileInputStream("/Users/mas939/Desktop/dasu.jks");
	         ks.load(is, "fadazani".toCharArray());
	         
	         ArrayList<X509Certificate> certs = new ArrayList();
	         Enumeration<String> en = ks.aliases();
	         for (; en.hasMoreElements(); ) {
	             String alias = (String)en.nextElement();
	             if (ks.isCertificateEntry(alias)) {
	            	 certs.add((X509Certificate) ks.getCertificate(alias));
	             }
	         }

	         JarVerifier.verify(jf, certs.toArray(new X509Certificate[]{}));
	         //JarVerifier.verify(jf, new X509Certificate[]{(X509Certificate) ks.getCertificate("rulesuimodules.aqualab.cs.northwestern.edu")});	        
	         //JarVerifier.verify(jf, new X509Certificate[]{(X509Certificate) ks.getCertificate("reports.aqualab.cs.northwestern.edu")});
    		
	         jf = new JarFile("/tmp/DasuUiRulesModules.jar");
	         ZipEntry ze = jf.getEntry("probe.modules.properties");
	         is = jf.getInputStream(ze);
	         Properties prop = new Properties();
	         prop.load(is);
    		
	         for (String key : prop.stringPropertyNames()){
    			System.out.println(key + " " + prop.getProperty(key));
	         }
    		
    		
    	}
    		
    	catch (CertificateException e) {
    		e.printStackTrace();
    	} catch (KeyStoreException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		*/
    	

    	
    	 
    }
}