/**
 * Dasu Project
 *
 * File:         Jsonizable.java
 * RCS:          $Id: Jsonizable.java,v 1.0 Feb 7, 2013 1:31:05 PM mas939 Exp $
 * Description:  Jsonizable class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 7, 2013 at 1:31:05 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.util
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2013, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import edu.northwestern.cs.aqualab.cattle.jsonsimple.JSONObject;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The Jsonizable class ...
 */
public abstract class Jsonizable {

	/* By default, jsonize every public field 
	 * 
	 * Explicitly add this method to expose protected fields and remove
	 * public ones you don't want to jsonize
	 *  
	 */

	public JSONObject toJSON() {	
		
		JSONObject json = new JSONObject();
		
		try {
			
			for (Field f : this.getClass().getFields()) {
				
				if ( f.get(this) != null ) {
					
					Class<? extends Object> clazz = f.get(this).getClass();
					if (Jsonizable.class.isAssignableFrom(clazz)) {
											
						try {
							Method m = clazz.getMethod("toJSON", new Class<?>[]{});
							json.put(f.getName(), m.invoke(f.get(this), new Object[]{}));
						} catch (SecurityException e) {
							e.printStackTrace();
						} catch (NoSuchMethodException e) {
							e.printStackTrace();
						} catch (InvocationTargetException e) {
							e.printStackTrace();
						}
						
					}
				}
				
				Object val = f.get(this);
				if ( val != null && val.getClass().isEnum())
					val = val.toString();
				
				if ( !json.containsKey(f.getName()) )
					json.put(f.getName(), val);
											
			}
			
			
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}	
		
		return json;
	}
	
}
