/**
 * Dasu Project
 *
 * File:         Util.java
 * RCS:          $Id$
 * Description:  Util class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Aug 28, 2006 at 11:21:35 AM
 * Language:     Java
 * Package:      edu.northwestern.news.util
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.util;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.URI;
import java.net.URL;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.nio.LongBuffer;
import java.nio.ShortBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import java.util.TimeZone;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.xbill.DNS.DClass;
import org.xbill.DNS.ExtendedFlags;
import org.xbill.DNS.Message;
import org.xbill.DNS.Name;
import org.xbill.DNS.Record;
import org.xbill.DNS.ReverseMap;
import org.xbill.DNS.Section;
import org.xbill.DNS.SimpleResolver;
import org.xbill.DNS.TSIG;
import org.xbill.DNS.TextParseException;
import org.xbill.DNS.Type;

import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.measurement.types.*;
import edu.northwestern.dasu.signal.statistics.SignalManipulation;



/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The Util class provides utility functions for the package.
 */
public class Util {
	
	private final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	    
	static BufferedReader in;

	private static Util self;
    
	public static long dasuTimeOffset = 0;
	public static long sntpOffset = 0;
	
	private static boolean failed = false;
    
    public static interface PingResponse {
    	public void response(double rtt);
    }

    @SuppressWarnings("deprecation")
	public static String currentGMTDayHour() {
    	Date d = new Date(currentGMTTime());
    	String dayHour = d.getDay() + "-" + d.getHours(); 
    	return dayHour;
    }
    
    public static long currentGMTTime(){
    	
    	if ( Main.initialized ) {
	    	if (DasuManager.USE_SIMULATION_V2 && DasuManager.interferenceSimulator != null)
	    		return DasuManager.interferenceSimulator.currentGMTTime();
    	}
    		
    	return System.currentTimeMillis()-dasuTimeOffset;
    	//return currentDasuTime();
    	//return System.currentTimeMillis()-TimeZone.getDefault().getRawOffset();
    	
    } 

    public static long currentTimeMillis(){
	    
    	if (Main.initialized  && DasuManager.USE_SIMULATION_V2 && DasuManager.interferenceSimulator != null) 
		 		return DasuManager.interferenceSimulator.currentGMTTime();
    	
    	
    	return System.currentTimeMillis();
    }

    public static long currentDasuTime(){
    	return currentGMTTime() + TimeZone.getDefault().getRawOffset();
    }
    
    public static long currentGMTTimeV2() {
    	
    	return System.currentTimeMillis() + sntpOffset;
    }
    
    public static long currentDasuTimeV2() {
    	
    	return System.currentTimeMillis() + dasuTimeOffset;
    	
    }
    
    public static long fromDasuTime(long dasuTime){
    	return dasuTime+dasuTimeOffset;
    }
    
    public static void setDasuTimeOffset( long offset ){
    	
    	if (offset == -1)
    		failed = true;
    	
    	dasuTimeOffset = offset + TimeZone.getDefault().getRawOffset();
    	sntpOffset = offset;
    }
    
    public static long getDasuSntpTimeOffset() {
    	return sntpOffset;
    }
    
    public static long getDasuTimeOffset() {
    	return dasuTimeOffset;
    }
    
    public static long fromGMTTime(long gmtTime){
    	return gmtTime+TimeZone.getDefault().getRawOffset();
    }
    
    public synchronized static Util getInstance(){

	    	if (self == null){
	    		self = new Util();
	    	}
    	
    	return self;
    }
    
    public static boolean isFailed() {
    	return failed;
    }

   

    /**
     * @param newEdge
     * @return
     */
    public static String getClassCSubnet(String ipaddress) {
        // TODO Auto-generated method stub
        return ipaddress.substring(0, ipaddress.lastIndexOf("."));
    }

	public static byte[] convertLong(long l) {
		byte[] bArray = new byte[8];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        LongBuffer lBuffer = bBuffer.asLongBuffer();
        lBuffer.put(0, l);
        return bArray;
	}
	
	public static  byte[] convertShort(short s) {
		byte[] bArray = new byte[2];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        ShortBuffer sBuffer = bBuffer.asShortBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertInt(int s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        IntBuffer sBuffer = bBuffer.asIntBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertFloat(float s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        FloatBuffer sBuffer = bBuffer.asFloatBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static long byteToLong(byte data[]){
		ByteBuffer bBuffer = ByteBuffer.wrap(data);
		bBuffer.order(ByteOrder.LITTLE_ENDIAN);
		LongBuffer  lBuffer = bBuffer.asLongBuffer();
		return lBuffer.get();
	}

	public static byte[] convertStringToBytes(String key) {
		try {
			return key.getBytes("ISO-8859-1");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	public static String convertByteToString(byte[] value){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			baos.write(value);
		
			return baos.toString("ISO-8859-1");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}

	public static String convertIntToIp(int ip) {
		int temp;
		String out = "";
		for (int i = 0; i < 4; i++){
			temp = ip;
			temp >>= (8*(3-i));
			temp &= 0xFF;
			out+=temp+".";
		}
		return out.substring(0, out.length()-1);
	}

	public static int convertIpToInt(String ipv4) {
		String parts[] = ipv4.split("\\.");
		if (parts.length!=4) return -1;
		long ip = 0;
		long temp;
		for (int i = 0; i < parts.length; i++){
			temp = Integer.parseInt(parts[i]);
			temp <<= (8*(3-i));
			ip+=temp;
		}
		return (new Long(ip)).intValue();
	}

	public static String join(String string, Object[] array) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < array.length; i++){
			sb.append(array[i]);
			if (i<array.length-1) sb.append(string);
		}
		return sb.toString();
	}
	
	public static byte[] hexStringToByteArray(String s) {
		
		byte[] b = new byte[s.length() / 2];
	    for (int i = 0; i < b.length; i++){
	    	int index = i * 2;
	    	int v = Integer.parseInt(s.substring(index, index + 2), 16);
	    	b[i] = (byte)v;
	    }
	    return b;
	}
	
	public static String getHexString(byte[] b) {
		String result = "";
		for (int i=0; i < b.length; i++) {
			result +=
		          Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 );
		}
		return result;
	}
	
	/**
	 * Does a remote lookup. If the -x option is specified, only authoritative
	 * name servers are returned. Otherwise, an array of resolved IP addresses
	 * is returned
	 * 
	 * @param argv
	 *            the dig command parameterse
	 * @return
	 * @throws IOException
	 */
	public static Record[] doLookup(String[] argv)  {

		org.xbill.DNS.Name name = null;
		int type = org.xbill.DNS.Type.A;
		int dclass = DClass.IN;
		String server = null;
		int arg;
		Message query;
		Message response = null;
		Record rec;
		SimpleResolver res = null;
		boolean printQuery = false;

		if (argv.length < 1) {
			throw new RuntimeException("Invalid dig format!");
		}

		try {
			arg = 0;

			if (argv[arg].startsWith("@")) {
				server = argv[arg++].substring(1);
				res = new SimpleResolver(server);
			}

			String nameString = argv[arg++];

			if (nameString.equals("-x")) {
				name = ReverseMap.fromAddress(argv[arg++]);
				type = Type.PTR;
				dclass = DClass.IN;
			} else {
				name = Name.fromString(nameString, Name.root);
				type = Type.value(argv[arg]);

				if (type < 0) {
					type = Type.A;
				} else {
					arg++;
				}

				dclass = DClass.value(argv[arg]);

				if (dclass < 0) {
					dclass = DClass.IN;
				} else {
					arg++;
				}
			}

			while (argv[arg].startsWith("-") && (argv[arg].length() > 1)) {
				switch (argv[arg].charAt(1)) {
				case 'p':

					String portStr;
					int port;

					if (argv[arg].length() > 2) {
						portStr = argv[arg].substring(2);
					} else {
						portStr = argv[++arg];
					}

					port = Integer.parseInt(portStr);

					if ((port < 0) || (port > 65536)) {
						LOGGER.warning("Invalid port");

						return null;
					}

					res.setPort(port);

					break;

				case 'b':

					String addrStr;

					if (argv[arg].length() > 2) {
						addrStr = argv[arg].substring(2);
					} else {
						addrStr = argv[++arg];
					}

					InetAddress addr;

					try {
						addr = InetAddress.getByName(addrStr);
					} catch (Exception e) {
						LOGGER.warning("Invalid address");

						return null;
					}

					res.setLocalAddress(addr);

					break;

				case 'k':

					String key;

					if (argv[arg].length() > 2) {
						key = argv[arg].substring(2);
					} else {
						key = argv[++arg];
					}

					res.setTSIGKey(TSIG.fromString(key));

					break;

				case 't':
					res.setTCP(true);

					break;

				case 'i':
					res.setIgnoreTruncation(true);

					break;

				case 'e':

					String ednsStr;
					int edns;

					if (argv[arg].length() > 2) {
						ednsStr = argv[arg].substring(2);
					} else {
						ednsStr = argv[++arg];
					}

					edns = Integer.parseInt(ednsStr);

					if ((edns < 0) || (edns > 1)) {
						LOGGER.info("Unsupported " + "EDNS level: " + edns);

						return null;
					}

					res.setEDNS(edns);

					break;

				case 'd':
					res.setEDNS(0, 0, ExtendedFlags.DO, null);

					break;

				case 'q':
					printQuery = true;

					break;

				default:
					LOGGER.info("Invalid option: " + argv[arg]);
				}

				arg++;
			}
		} catch (ArrayIndexOutOfBoundsException e) {
			if (name == null) {
				// log.log("Name is null!");
				throw new RuntimeException("Name is null!");
			}
		} catch (UnknownHostException e) {
			LOGGER.warning(e.toString());
		} catch (TextParseException e) {
			LOGGER.warning(e.toString());
		}

		if (res == null) {
			try {
				res = new SimpleResolver();
			} catch (UnknownHostException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				LOGGER.warning(e.toString());
			}
		}

		if (name == null) {
			return null;
		}

		rec = Record.newRecord(name, type, dclass);
		query = Message.newQuery(rec);

		if (printQuery) {
			System.out.println(query);
		}

		try {
			response = res.send(query);
		}
		catch (Exception e) {
			//timeout, do nothing
		}

		// return only the authority section
		if ((response != null) && argv[0].equals("-x")) {
			if (argv.length == 2)
				return response.getSectionArray(Section.ANSWER);
		}

		// return the actual ip addresses
		if (response != null) {
			return response.getSectionArray(Section.ANSWER);
		}

		return null;
	}
	
	
	 private static final String ipv4Pattern = "(([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.){3}([01]?\\d\\d?|2[0-4]\\d|25[0-5])";
	 private static final String ipv6Pattern = "([0-9a-f]{1,4}:){7}([0-9a-f]){1,4}";
	 private static Pattern VALID_IPV4_PATTERN = Pattern.compile(ipv4Pattern, Pattern.CASE_INSENSITIVE);
	 private static Pattern VALID_IPV6_PATTERN = Pattern.compile(ipv6Pattern, Pattern.CASE_INSENSITIVE);

	  /**
	   * Determine if the given string is a valid IPv4 or IPv6 address.  This method
	   * uses pattern matching to see if the given string could be a valid IP address.
	   *
	   * @param ipAddress A string that is to be examined to verify whether or not
	   *  it could be a valid IP address.
	   * @return <code>true</code> if the string is a value that is a valid IP address,
	   *  <code>false</code> otherwise.
	   */
	 
		public static boolean isValidIp(final String ipAddress) {
									
		    Matcher m1 = VALID_IPV4_PATTERN.matcher(ipAddress);
		    //System.out.println("PING: isValidIp " + ipAddress + m1.matches());
		    if (m1.matches()) {
		      return true;
		    }
		    Matcher m2 = VALID_IPV6_PATTERN.matcher(ipAddress);
		    //System.out.println("PING: isValidIp " + ipAddress + m2.matches());
		    return m2.matches();
	  }
		
	  public static boolean isValidInet4Address(String ipAddress) {

		    Matcher m1 = VALID_IPV4_PATTERN.matcher(ipAddress);
		    if (m1.matches()) {
		      return true;
		    }
		    return false;
	  }
	  
	  public static boolean isValidInet6Address(String ipAddress) {
		  
		  Matcher m1 = VALID_IPV6_PATTERN.matcher(ipAddress);
		  if (m1.matches()) {
			  return true;
		  }
		  return false;
	  }
	  
	  //Check if given string is numeric
	  public static boolean isNumeric(String string) {
	      return string.matches("^[-+]?\\d+(\\.\\d+)?$");
	  }

	  //Check if a dynamic probeType exists
	  public static boolean probeTypeExists(String probeType) {
			
			try {
				ProbeType.valueOf(probeType);
				return true;
			}
			catch(Exception e){
				return false;
			}
			
	  }
	  


	  public static String toUtf8(String s) {
		  
		  try {
			  byte[] utf8Bytes = s.getBytes("UTF8");
			  return new String(utf8Bytes, "UTF8");
		  }
		  catch (Exception e) {
			  LOGGER.warning(e.getMessage());
			  return s;
		  }
	  }
	  
	  /**
	   * True if character is a Greek letter.
	   * 
	   * @param c
	   *            Character to check for being a Greek letter.
	   * 
	   * @return true if character is a Greek letter.
	   */

	  public static boolean isGreekLetter(char c) {
	    return (((c >= 0x0370) && (c < 0x0400)) || ((c >= 0x1f00) && (c < 0x2000)));
	  }

	  /**
	   * True if any characters in a string are Greek letters.
	   * 
	   * @param s
	   *            String to check for Greek letters.
	   * 
	   * @return true if any characters are Greek letters.
	   */

	  private static boolean hasGreekLetters(String s) {
	    boolean result = false;

	    String ts = s.trim();

	    for (int i = 0; i < ts.length(); i++) {
	      char ch = ts.toUpperCase().charAt(i);

	      if (isGreekLetter(ch)) {
	        result = true;
	        break;
	      }
	    }

	    return result;
	  }	  
	  
	  public static String removeGreekLetters(String s) {
	    
		  String ts = toUtf8(s).trim();
		  StringBuilder result = new StringBuilder();
	    
		  for (int i = 0; i < ts.length(); i++) {
			  char ch = ts.toUpperCase().charAt(i);

			  if (!isGreekLetter(ch))
				  result.append(s.charAt(i));
			  else
				  result.append("_");
			  
		  }

		  return result.toString();
	  }	 
	  
	  public static void main(String args[]) throws Exception {
		  
		  
		  System.out.println(java.awt.Desktop.isDesktopSupported());
		  java.awt.Desktop desktop = java.awt.Desktop.getDesktop();
		  URI url = new URI("http://www.google.com");
		  desktop.browse(url);  
		  System.exit(0);
		  
		  
		  Float[] d = new Float[]{-0.0f,1.5491128f,1.5308714f,4.670493f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5568911f,40.275845f,56.1007f,0.0f,0.0f,1.5568442f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,38.76474f,0.0f,1.5582122f,45.065197f,70.12697f,1.5564082f,74.52601f,73.23209f,1.5586451f,1.5536151f,1.556958f,0.0f,0.0f,42.07593f,0.0f,1.5506738f,0.0f,0.0f,37.25247f,90.389404f,80.95524f,6.208676f,38.96046f,52.98373f,17.073689f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5583848f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,4.6565866f,49.820267f,52.9857f,1.5537308f,6.233589f,65.45292f,58.228977f,0.0f,0.0f,0.0f,1.5599205f,1.5552956f,0.0f,0.0f,0.0f,0.0f,1.5568293f,0.0f,0.0f,1.558392f,26.492435f,56.531567f,46.751656f,6.233702f,0.0f,0.0f,1.5567697f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.558343f,0.0f,12.471476f,73.17337f,85.45661f,82.593575f,81.03781f,77.68352f,62.274803f,51.428318f,55.93462f,32.724503f,0.0f,24.786139f};
		  System.out.println(Arrays.asList(d));
		  System.out.println(SignalManipulation.getAverage(Arrays.asList(d)));
		  System.exit(0);
		  		  
		  Properties p = System.getProperties();
		  Enumeration iter = p.keys();
		  while (iter.hasMoreElements()) {			  
			  System.out.println(iter.nextElement());
		  }
		  
			//String filename = "/Users/mas939/Mario's Stuff/NORTHWESTERN/Dropbox/workspace/ISP Interference/testdasu/plugins/dasu/reports/report-b322b835-fa94-426f-a125-cd32a90c75fa";
			String filename = "/Users/mas939/Mario's Stuff/NORTHWESTERN/Dropbox/workspace/ISP Interference/testdasu/plugins/dasu/reports/uuid";
				
				ObjectInputStream objstream;
				Object object = null; 

				try {
					File f = new File(filename);
					if (f.isFile()) {
						objstream = new ObjectInputStream(new FileInputStream(f));
						object = objstream.readObject();
						objstream.close();				
					}
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					//e.printStackTrace();
					//when versionID for persistenData() class changed, then don't load,
					//ignore error and simply override file
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}

				System.out.println(object);
			

	  }

	
}

